<?php
// Administrate web integration 
// TODO: Consolidate class AdministrateWeblink and class AdminsitratePlugin into this class
//
abstract class Administrate {

	// Get event log
	public static function log() {
		return AdministrateEventLog::singleton();
	}
	
	// Get API object
	public static function api() {
		return AdministrateAPI::singleton();
	}

	/**
	 * Returns the required $text in the language required.
	 *
	 * @param $text string - The text to be translated.
	 * @return string - The translated text.
	 */
	abstract public function get_text($text);

	/**
	 * Echoes out the required $text in the language required.
	 *
	 * @param $text string - The text to be translated.
	 * @return null
	 */
	abstract public function echo_text($text);

	// Date
	public function t_date( $date, $show_month = true, $show_year = true ) {
		$date_format = strtolower( $this->get_option('date_format', 'event') );
		return AdministrateDateTime::date( $date, $date_format, $show_month, $show_year );
	}

	public function render_datetime_string($datetime_string) {
		$datetime = new DateTime($datetime_string);
		return $datetime->format('d M Y');
	}

	public function showChosenObjectiveDetailsIfExist($selectedObjectives, $objective){
		$event_id = $this->findEventId($selectedObjectives, $objective);
		foreach ($objective->registrable_events as $event) {
			if ($event_id == $event->id) {
				return " - ".$event->location." - ".$this->render_datetime_string($event->date);
			}
		}
	}
	
	// Date range
	public function t_dates( $dates, $online_event=false, $persistent_event=false ) {
		$show_current_online = $this->get_bool_option('show_current_online', 'event');
		$date_format = strtolower( $this->get_option('date_format', 'event') );
		$show_year = $this->get_bool_option('show_year', 'event');

		if($persistent_event) {
			return '';
		}

		if( $show_current_online && $online_event ) {
			return $this->get_text("Online");
		}
		return AdministrateDateTime::date_range( $dates, $date_format, $show_year );
	}
	
	// Time
	public function t_time($time, $persistent_event=false) {
		if($persistent_event) {
			return null;
		}
		$show_24h_times = $this->get_bool_option('show_24h_times', 'event');
		return AdministrateDateTime::time( $time, $show_24h_times );
	}
	
	// Time range
	public function t_times($times) {
		$show_24h_times = $this->get_bool_option('show_24h_times', 'event');
		return AdministrateDateTime::time_range( $times, $show_24h_times );
	}
	
	// Price
	abstract public function t_prices(
		$event, $basis = false,
		$currency='',
		$show_currency_indicator=true, $currency_indicator='',
		$currency_list=false
	);
	
	/*
		Options
	*/
	
	// Get option 
	abstract public function get_option($name, $group, $sub_group=null);

	/**
	 * Gets an option as a boolean. Useful in weblink where params may be scraped as strings.
	 *
	 * @param $name string - The option to lookup.
	 * @param null $group string - The group of the option.
	 * @param null $sub_group - (optional) The sub group of the option.
	 * @return bool
	 */
	abstract public function get_bool_option($name, $group, $sub_group = null);
	
	// Get integer option 
	abstract public function get_int_option($name, $group, $sub_group = null);
	
	// Is object hidden
	abstract public function is_hidden($object_type, $id);
		
	/*
		Widgets
	*/
	
	// Course widget url
	abstract public function get_widget_url_course($course_id);

	// Checkout widget url
	abstract public function get_widget_url_checkout($event_id, $event_currency=false);

	abstract public function get_registration_url($eventId=0, $currency=false, $track_id=null);

	// Track url
	abstract public function get_widget_url_track($track_id = false);

	// Track List
	abstract public function get_widget_url_track_list();
	
	// Resource url
	abstract public function get_resource_url($resource_file);
	
	// Directory of widgets common to Wordpress and Weblink
	public function get_widget_dir($widget_name=null) {
		$dir = dirname(__FILE__) . "/widgets";
		
		if($widget_name) {
			return $dir. "/" .$widget_name;
		}
		return $dir;
	}

	abstract public function get_product();

	/**
	 * Parses PHP's $_POST global to check whether or not pay_by_invoice_selected is present, and if it is, returns
	 * it's contents as a boolean. Defaults to false.
	 *
	 * @return bool
	 */
	public static function get_is_invoice_payment_from_post() {
		if(array_key_exists('pay_by_invoice_selected', $_POST)) {
			return ($_POST['pay_by_invoice_selected'] == '1');
		}

		// This is an arbitrary choice. If pay_by_invoice_selected doesn't exist in $_POST, we default to not invoice.
		return false;
	}

	protected function findEventId($selectedObjectives, $objective) {
		$split = explode(',', $selectedObjectives);
		foreach ($split as $item) {
			$split_objective = explode('-', $item);
			$objective_id = $split_objective[0];
			$event_id = $split_objective[1];
			if ($objective_id == $objective->id) {
				return $event_id;
			}
		}
	}
}
