<?php
// Administrate event log
//
class AdministrateEventLog {

	// Single-instance event log
	public static function singleton() {
		static $event_log = null;

		if( is_null($event_log) ) {
			$event_log = new AdministrateEventLog();
		}

		return $event_log;
	}

	// Construct event log
	protected function __construct() {
		$this->_event_logs = array();
		$this->_filter_error_level = E_ALL;
		$this->_debug = false;
	}

	// Set/get debug flag
	//
	public function set_debug($debug) {
		$this->_debug = $debug;
	}

	public function get_debug() {
		return $this->_debug;
	}

	// Clear event log
	public function clear($error_level) {
		$this->_event_logs[$error_level] = array();
	}

	// Get event log for the given error level
	public function get($error_level, $extract_messages = true) {

		$this->_filter_error_level = $error_level;
		$filtered_logs = array_filter( $this->_event_logs, array($this, 'log_entry_filter') );
		if( $extract_messages ) {
			return array_map( array($this, 'log_entry_message'), $filtered_logs );
		}
		return $filtered_logs;
	}

	public function log_entry_filter($log_entry) {
		if( $this->_filter_error_level == E_ALL ) {
			return true;
		}
		if(array_key_exists('error_level', $log_entry) && $log_entry['error_level'] == $this->_filter_error_level) {
			return true;
		}
		return false;
	}

	public function log_entry_message($log_entry) {
		if(array_key_exists('message', $log_entry)) {
			return $log_entry["message"];
		}
		return '';
	}

	// Log user message
	//
	public function notice($user_message) {
		$this->_log($user_message, E_NOTICE);
	}

	public function warning($user_message) {
		$this->_log($user_message, E_WARNING);
	}

	public function error($user_message) {
		$this->_log($user_message, E_ERROR);
	}

	// Internal log function
	protected function _log($user_message, $error_level) {
		switch($error_level) {
			case E_ERROR:
				$type_str = "[Error]"; break;
			case E_NOTICE:
				$type_str = "[Notice]"; break;
			case E_WARNING:
				$type_str = "[Warning]"; break;
			default:
				$error_level = E_NOTICE;
				$type_str = "[Notice]"; break;
		}

		$timestamp = date("d-m-Y H:i:s");
		$msg_str = $timestamp . " " . $type_str . " : " . $user_message;

		$this->_log_entry($msg_str, $error_level);

		if ( $this->get_debug() ) {
			error_log($msg_str);
		}
	}

	private function _log_entry($msg_str, $error_level) {
		$entry = array( "message" => $msg_str, "error_level" => $error_level);
		array_push( $this->_event_logs, $entry );
	}
}

?>
