<?php
abstract class AdministrateAPIObject {
	public function __construct($data_fields = false) {
		$this->data_fields = $data_fields;
	}
	
	//  Prepare fields for API call
	public static function prepare_api_fields($inputFields = array()) {
		$caller_class = get_called_class();
		$outputFields = array();
		
		foreach ($caller_class::_get_api_map() as $field=>$properties) {
			if (array_key_exists($field, $inputFields)) {
				$api_field = $properties['api_field'];
				$value = $inputFields[$field];
				
				//  If a value mapping was specified, apply it
				if (isset($properties['mappings'])) {
					$value = $properties['mappings'][$value];
				}
				
				//  If a time format was specified, apply it
				if (isset($properties['time_format'])) {
					$value = date($properties['time_format'], $value);
				}
				
				//  Save it as an output field
				$outputFields[$api_field]  = $value;
			
			//  Or if the field doesn't exist, but the auto fill flag is set, use the default
			} 
			else if (isset($properties['auto_fill']) && $properties['auto_fill']) {
				$outputFields[$properties['api_field']] = $properties['default'];
			}

		}

		//  Return output fields
		return $outputFields;
		
	}
	
	//  Prepare object for API call
	public static function prepare_api_object($inputFields = array()) {
		$caller_class = get_called_class();
		$prepared_fields = $caller_class::prepare_api_fields($inputFields);
		
		$obj = new stdClass();
		foreach ($prepared_fields as $key=>$val) {
			$obj->$key = $val; 	
		}
		return $obj;
	}
	
	//  Get a field
	protected function _get_field($field_name) {
		$fields = $this->_get_api_map();
		$field_api_name = $fields[$field_name]['api_field'];

		if ($this->data_fields && property_exists($this->data_fields, $field_api_name)) {
			return $this->data_fields->$field_api_name;	
		} 
		return false;
	}
	
	//  Get the ID
	public function get_id() {
		return $this->_get_field('id');	
	}
	
	//  Whether the object exists or not
	public function exists() {
		return ($this->get_id() !== false);	
	}
	
}
?>
