<?php
//  Administrate API User
class AdministrateAPIUser extends AdministrateAPIObject {
	
	private static $fields = array(
		'id'			=> array(
			'api_field'	=>	'WebsiteUserID'
		),
		'is_individual'	=> array(
			'api_field'	=>	'IsIndividual',
			'default'	=>	0,
			'required'	=>	true
		),
		'email'	=> array(
			'api_field'	=>	'Email',
			'required'	=>	true
		),
		'no_contact'	=> array(
			'api_field'	=>	'DontMail',
			'default'	=>	0,
			'required'	=>	true
		),
		'password'	=> array(
			'api_field'	=>	'Password'
		),
		'company'	=> array(
			'api_field'	=>	'Company',
			'required'	=>	true
		),
		'first_name'	=> array(
			'api_field'	=>	'FirstName',
			'required'	=>	true
		),
		'last_name'	=> array(
			'api_field'	=>	'LastName',
			'required'	=>	true
		),
		'title'	=> array(
			'api_field'	=>	'JobTitle'
		),
		'department'	=> array(
			'api_field'	=>	'Department'
		),
		'address1'	=> array(
			'api_field'	=>	'Address1'
		),
		'address2'	=> array(
			'api_field'	=>	'Address2',
		),
		'address3'	=> array(
			'api_field'	=>	'Address3'
		),
		'city'	=> array(
			'api_field'	=>	'City'
		),
		'territory'	=> array(
			'api_field'	=>	'County'
		),
		'postal_code'	=> array(
			'api_field'	=>	'PostCode'
		),
		'country'	=> array(
			'api_field'	=>	'CountryCode'
		),
		'phone'	=> array(
			'api_field'	=>	'Tel'
		),
		'mobile'	=> array(
			'api_field'	=>	'Mobile'
		),
		'default_currency'	=> array(
			'api_field'	=>	'DefaultCurrency'
		),
		'referred_by'	=> array(
			'api_field'	=>	'HearAboutUs'
		),
		'referred_by_text'	=> array(
			'api_field'	=>	'HearAboutUsText'
		),
		'contact_id'	=> array(
			'api_field'	=>	'ContactID'
		),
		'custom_date1'	=> array(
			'api_field'	=>	'CustomDate1'
		),
		'custom_date2'	=> array(
			'api_field'	=>	'CustomDate2'
		),
		'custom_dropdown'	=> array(
			'api_field'	=>	'CustomDropDown1'
		),
		'delegates'	=> array(
			'api_field'	=>	'Delegates'
		)
	);
	
	// Get copy of fields
	protected static function _get_api_map() {
		return self::$fields;
	}
	
	//  Edit the user
	public function edit($fields = array()) {
			$fields['id'] = $this->get_id();	
			Administrate::log()->notice('Edit user ID ' . $fields['id'] . ' successful');
			return Administrate::api()->raw_call('editUser', $this->prepare_api_fields($fields));
	}
	
	//  Whether or not a delegate exists
	public function delegate_exists($email) {
		$this->_set_delegate_emails();
		return in_array($email, $this->delegateEmails);
	}
	
	//  Get a delegate by email address
	public function get_delegate_by_email($email) {
 		Administrate::log()->notice('Get delegate by email: ' . var_export($email,true));
		
 		// If no email, return immediately
 		if( !$email || ($email=="") ) {
 			return false;
 		}
		
		$this->_set_delegate_emails();
		$key = array_search($email, $this->delegateEmails);
		
		// No matching delegate
		if ($key === false) {
			return false;	
		}
		
		// Return match
		return $this->delegates[$key];
		
	}
	
	//  Set delegates
	public function get_delegates() {
		if (property_exists($this, 'delegates')) {
			return $this->delegates;
		}
		
		$delegates = $this->_get_field('delegates');
		for ($i = 0, $numDelegates = count($delegates); $i < $numDelegates; ++$i) {
			$delegates[$i] = new AdministrateAPIDelegate($delegates[$i]);	
		}
		
		$this->delegates = $delegates;
		return $this->delegates;
	}
	
	//  Set the delegate emails in an array
	protected function _set_delegate_emails() {
		if (!property_exists($this, 'delegateEmails')) {
			$this->get_delegates();
			$this->delegateEmails = array();
			foreach	($this->delegates as $delegate) {
				array_push($this->delegateEmails, $delegate->get_email());
			}
		}
	}
}
