/*
    Stripe payment form validation
*/
function administrate_initialize_stripe($) {
    // Token request
    //
    var validation_error = function( message, field_selector ) {
        var obj = $(".administrate-cc-validation");

        obj.html( message );
        obj.slideDown();

        if(field_selector) {
            var obj_highlight = $(field_selector);
            obj_highlight.addClass("administrate-stripe-invalid");
            setTimeout(function() {
                obj_highlight.removeClass("administrate-stripe-invalid");
            }, 1500);
        }
    };

    var stripe_public_key = $('#administrate-stripe-public-key').val();
    try {
        var stripe = Stripe(stripe_public_key);
        // Create an instance of Elements.
        var elements = stripe.elements();
        // Create an instance of the card Element.
        var style = {
            base: {
                color: '#32325d',
                fontSmoothing: 'antialiased',
                fontSize: '16px',
                '::placeholder': {
                    color: '#aab7c4'
                }
            },
            invalid: {
                color: '#fa755a',
                iconColor: '#fa755a'
            }
        };

        var card = elements.create('card', {style: style} );
        // Add an instance of the card Element into the `card-element` <div>.
        card.mount('#card-element');

        // Handle real-time validation errors from the card Element.
        card.addEventListener('change', function(event) {
            var displayError = document.getElementById('card-errors');
            if (event.error) {
                displayError.textContent = event.error.message;
            } else {
                displayError.textContent = '';
            }
        });

        // Handle form submission.
        $("#stripe-submit").click(function(event){
            $(".administrate-spinner").show();
            stripe.createToken(card).then(function(result) {
                if (result.error) {
                    // Inform the user if there was an error.
                    var errorElement = document.getElementById('card-errors');
                    errorElement.textContent = result.error.message;
                    $(".administrate-spinner").hide();
                } else {
                    // Send the token to your server.
                    stripeTokenHandler(result.token);
                }
            });
        });

        // Submit the form with the token ID.
        function stripeTokenHandler(token) {
            // Insert the token ID into the form so it gets submitted to the server
            var form = document.getElementById('administrate-checkout-form');
            var hiddenInput = document.createElement('input');
            hiddenInput.setAttribute('type', 'hidden');
            hiddenInput.setAttribute('name', 'stripe_token');
            hiddenInput.setAttribute('value', token.id);
            form.appendChild(hiddenInput);
            // Set the step to the begin Stripe step to avoid returning the whole page
            $("input[name=step]").val("8");
            // This works for Wordpress
            $("input[name=administrate_checkout_step]").val("8");
            $.ajax({
                type: "POST",
                url: form.action,
                data: $("#administrate-checkout-form").serialize(),
                success: function(resp) {
                    var jsonFlag = '<!-- ADMINISTRATE_JSON_START -->';
                    var jsonFlagEnd = '<!-- ADMINISTRATE_JSON_END -->';
                    if(resp.indexOf(jsonFlag) > 0) {
                        resp = resp.substr(resp.indexOf(jsonFlag) + jsonFlag.length);
                        if(resp.indexOf(jsonFlagEnd) > 0) {
                            resp = resp.substr(0, resp.indexOf(jsonFlagEnd));
                        }
                    }
                    var response = JSON.parse(resp);
                    if (response.error) {
                        // Show error from server on payment form
                        $(".administrate-spinner").hide();
                        $("#card-errors").text("There was a problem with these card details");
                        console.log(response.error);
                    } else if (response.requires_action) {
                        // Use Stripe.js to handle required card action
                        handleAction(response);
                    } else {
                        $(".administrate-spinner").show();
                        var form = document.getElementById('administrate-checkout-form');
                        $("input[name=step]").val("5");
                        $("input[name=administrate_checkout_step]").val("5");
                        form.submit();
                    }
                },
                done: function(resp) {
                    // Show error from server on payment form
                    $("#card-errors").text(
                        "There was a problem with these card details. Please check the details are correct"
                    );
                    console.log(resp);
                    $(".administrate-spinner").hide();
                }
            });
        }

        function handleAction(response) {
            $(".administrate-spinner").hide();
            stripe.confirmCardPayment(
                response.payment_intent_client_secret
            ).then(function(result) {
                if (result.error) {
                    // Show error from server on payment form
                    $("#card-errors").text("There was a problem with the Strong Customer Authentication");
                    console.log(response.error);
                } else {
                    $(".administrate-spinner").show();
                    var form = document.getElementById('administrate-checkout-form');
                    $("input[name=step]").val("5");
                    $("input[name=administrate_checkout_step]").val("5");
                    form.submit();
                }
            });
        }

    }
    catch(e) {
        validation_error("Invalid Stripe Integration", null);
        return false;
    }
    return false;

}
