<?php
// Base class for event Table, Slider and List
//
abstract class AdministrateEventWidget2 extends AdministrateWidget2 {
	public function run($widget_params = array()) {
		$this->configure($widget_params);
		$this->fetch_events();
	}

	public function configure($widget_params) {

		// Parameters common to event Table, Slider and List
		$global_params = array(
			'currency'                       => $this->service->get_option('currency', 'pricing'),
			'currency_list'                  => $this->service->get_option('currency_list', 'pricing'),
			'currency_indicator'             => $this->service->get_option('currency_indicator', 'pricing'),
			'basis'                          => $this->service->get_option('basis', 'pricing'),
			'course_code'                    => $this->service->get_option('course_code', ''),
			'course'                         => $this->service->get_option('course', ''),
			'checkout_page'                  => $this->service->get_option('checkout_page', 'checkout'),
			'tax_label'                      => $this->service->get_option('tax_label', 'pricing'),
			'no_events_message'              => $this->service->get_option('no_events_message', 'event'),
			'error_message_no_events'        => $this->service->get_option('error_message_no_events', 'event'),
			'num_months'                     => $this->service->get_int_option('num_months', 'event'),
			'group_size'                     => $this->service->get_int_option('group_size', 'event'),
			'group_by'                       => $this->service->get_int_option('group_by', 'event'),
			'group_title_pre'                => $this->service->get_option('group_title_pre', ''),
			'group_title_post'               => $this->service->get_option('group_title_post', ''),
			'show_mobile_friendly'           => $this->service->get_bool_option('show_mobile_friendly', 'event'),
			'show_prices'                    => $this->service->get_bool_option('show_prices', 'event'),
			'show_venues'                    => $this->service->get_bool_option('show_venues', 'event'),
			'show_times'                     => $this->service->get_bool_option('show_times', 'event'),
			'show_sold_out'                  => $this->service->get_bool_option('show_sold_out', 'event'),
			'show_today'                     => $this->service->get_bool_option('show_today', 'event'),
			'show_current_online'            => $this->service->get_bool_option('show_current_online', 'event'),
			'show_currency_indicator'        => $this->service->get_bool_option('show_currency_indicator', 'pricing'),
			'show_empty_courses'             => $this->service->get_bool_option('show_empty_courses', 'course'),
			'show_remaining_places'          => $this->service->get_bool_option('show_remaining_places', 'event'),
			'show_classroom_dates'           => $this->service->get_bool_option('show_classroom_dates', 'event'),
			'show_dates'                     => $this->service->get_bool_option('show_dates', 'event'),
			'show_locations'                 => $this->service->get_bool_option('show_locations', 'event'),
			'show_codes'                     => $this->service->get_bool_option('show_codes', 'event'),
			'show_sessions'                  => $this->service->get_bool_option('show_sessions', 'event'),
			'show_session_titles'            => $this->service->get_bool_option('show_session_titles', 'event'),
			'show_categories'                => true,
			'show_names'                     => true,
			'category_id'                    => '',
			'category'                       => '',
			'cat_id'                         => '',
			'location_name'                  => '',
			'location'                       => ''
		);

		// We need to fetch the default in a different way from WP to WL, because in WP we can read straight off
		// the system control panel, but in weblink we need to use the standard 'option.'
		if($this->service->get_product() == 'wordpress') {
			$global_params['registration_deadline_timezone'] = get_option('timezone_string') ?: get_option('gmt_offset');
		} else {
			$global_params['registration_deadline_timezone'] = $this->service->get_option('registration_deadline_timezone');
		}

		if( function_exists("shortcode_atts") ) {
			// Merge global params with Wordpress shortcode params
			$this->params = shortcode_atts($global_params, $widget_params);
		}
		else {
			// Merge global params with Weblink widget params
			$this->params = array_merge($global_params, $widget_params);
		}
		
		$params_bool = array(
		'show_mobile_friendly', 'show_prices', 'show_times', 'show_remaining_places', 'show_sold_out',
		'show_today', 'show_current_online', 'show_currency_indicator', 'show_empty_courses',
		'show_remaining_places', 'show_categories', 'show_names', 'show_codes',
		'show_sessions', 'show_session_titles', 'show_classroom_dates'
		);
		
		foreach( $params_bool as $key ) {
			$this->params[$key] = $this->service->str_to_boolean( $this->params[$key] );
		}
		
		$this->params['num_months']  = intval($this->params['num_months']);
		$this->params['group_size']  = intval($this->params['group_size']);
		$this->params['course']      = urldecode(strtolower($this->params['course']));

		if(!is_array($this->params['course_code'])) {
			$this->params['course_code'] = urldecode(strtolower($this->params['course_code']));
		}

		if( !is_string( $this->params['group_by'] ) ) {
			$this->params['group_by'] = 'none';
		}
		
		// If currency list was passed as string, JSON decode it
		if( is_string( $this->params['currency_list'] ) ) {
			$this->params['currency_list'] = self::decode_currency_list( $this->params['currency_list'] );
		}
		
		// Currency list empty or not parsing
		if( !is_array($this->params['currency_list']) || empty($this->params['currency_list']) ) {
			$this->params['currency_list'] = false;
		}
		
		if( $this->params['currency_list'] ) {
			// Default currency is first on the list
			$this->params['currency'] = current(array_keys($this->params['currency_list']));
		}
		
		if( empty( $this->params['no_events_message']  ) ) {
			// Alias of the same parameter for compatibility
			$this->params['no_events_message'] = $this->params['error_message_no_events'];
		}
	
		//  Initialize the event filter
		// 
		// NOTE: The flag "show_prices" determines if prices are displayed in the template
		// Regardless of the value of this flag, prices should be included in the event filter,
		// otherwise there is no way of knowing what currencies are available for an event.
		//
		$event_filter = array(
		'IncludePrices'         => true,
		'IncludeCurrentOnline'  => $this->params['show_current_online'],
		'IncludeSessions'       => $this->params['show_sessions']
		);
		
		$cat_key       = 'administrate_event_table_category';
		$month_key     = 'administrate_event_table_month';
		$location_key  = 'administrate_event_table_location';

		if( !empty($this->params['course_code']) ) {
			$event_filter['CourseCode'] = $this->params['course_code'];
		}
		else if( !empty($this->params['course']) ) {
			// For compatibility with the old Wordpress event table
			// also accept 'course' for the same filter 
			$event_filter['CourseCode'] = $this->params['course'];
		}

		if(!function_exists('get_cat_filters')) {
			function get_cat_filters($cat_string) {
				$cat_subcat = explode(':', $cat_string); // cat:subcat

				$ret = array();
				$ret['CourseCategoryID'] = intval($cat_subcat[0]);

				if (array_key_exists(1, $cat_subcat)) {
					$ret['CourseSubCategoryID'] = intval($cat_subcat[1]);
				}

				return $ret;
			}
		}

		if( isset($_REQUEST[$cat_key]) && !empty($_REQUEST[$cat_key]) ) {
			$this->params['cat_id'] = $_REQUEST[$cat_key];
			$event_filter = array_merge($event_filter, get_cat_filters($_REQUEST[$cat_key]));
		}
		else if( $this->params['category_id'] ) {
			$this->params['cat_id'] = $this->params['category_id'];
			$event_filter = array_merge($event_filter, get_cat_filters($this->params['category_id']));
		}
		else if( $this->params['category'] ) {
			$this->params['cat_id'] = $this->params['category'];
			$event_filter = array_merge($event_filter, get_cat_filters($this->params['category']));
		}

		if( isset($_REQUEST[$month_key]) && !empty($_REQUEST[$month_key]) ) {
			$this->params['num_saved_months'] = $_REQUEST[$month_key];
		}
		else {
			$this->params['num_saved_months'] = $this->params['num_months'];
		}
		
		if( isset($_REQUEST[$location_key]) && !empty($_REQUEST[$location_key]) ) {
			$this->params['location_name'] = $_REQUEST[$location_key];
			$event_filter['EventLocationName'] = $this->params['location_name'];
		} else {
			if(!is_null($this->params['location'])) {
				$event_filter['EventLocationName'] = $this->params['location'];
			}
		}

		if($event_filter['EventLocationName'] === '') {
			unset($event_filter['EventLocationName']);
		}

		$this->params['event_filter'] = $event_filter;
	}
	
	public static function decode_currency_list($str) {
		$currency_list = json_decode( $str, true );
		
		// Wordpress settings page strips " in options
		// Make it valid JSON
		if( !$currency_list ) {
			return json_decode(preg_replace('/(\w+)/i', '"\1"', $str), true );
		}
		return $currency_list;
	}
	
	// Make API call with the active event filter
	public function fetch_events() {

		if(array_key_exists('CourseCode', $this->params['event_filter']) && !is_array($this->params['event_filter']['CourseCode'])) {
			$this->params['event_filter']['CourseCode'] = array_map('trim', explode(',', $this->params['event_filter']['CourseCode']));
		}

		$full_event_list = Administrate::api()->get_event_list($this->params['event_filter']);

		if( is_array($full_event_list) ) {
			$this->params['api_connection'] = true;
		}
		else {
			$this->params['api_connection'] = false;
			$full_event_list = array();
		}

		$this->params['event_list'] = array_filter( $full_event_list, array($this,"is_event_visible") );
	}

	private $_course_status = array();
	private $_category_status = array();
	private $_subcategory_status = array();

	/**
	 * Callback for the array_filter call above. Is a given event
	 * visible on the event 'list' widgets (list, table, etc.)
	 *
	 * @param $event AdministrateAPIEvent - The event to check.
	 * @return bool - Is the event visible?
	 */
	public function is_event_visible($event) {

		// If the event has a registration deadline assigned then ensure it's in the
		// future. If it's in the past then return false - the event should not be visible.
		if(!is_null($event->get_registration_deadline())) {
			$deadline = new DateTime($event->get_registration_deadline());
			$now = new DateTime();
			if($deadline < $now) {
				return false;
			}
		}

		// Event not visible, if course is hidden in the CEO settings
		if(!array_key_exists($event->get_course_code(), $this->_course_status)) {
			$this->_course_status[$event->get_course_code()] = $this->service->is_hidden("course", $event->get_course_code());
		}

		if($this->_course_status[$event->get_course_code()]) {
			return false;
		}
		
		// Event not visible, if all of its categories are hidden in the CEO settings
		$categories   = $event->get_categories();
		$visible_cats = count( $categories );
		
		foreach( $categories as $cat ) {

			if(!array_key_exists($cat->get_id(), $this->_category_status)) {
				$this->_category_status[$cat->get_id()] = $this->service->is_hidden("category", $cat->get_id());
			}

			if($this->_category_status[$cat->get_id()]) {
				$visible_cats--;
			}
			else {
				// Category could also be hidden if all subcategories are hidden
				$subcategories   = $cat->get_subcategories();
				$visible_subcats = count($subcategories);
				
				foreach( $subcategories as $subcat ) {

					if(!array_key_exists($subcat->get_id(), $this->_subcategory_status)) {
						$this->_subcategory_status[$subcat->get_id()] = $this->service->is_hidden("subcategory", $subcat->get_id());
					}

					if($this->_subcategory_status[$subcat->get_id()]) {
						$visible_subcats--;
					}
				}
				if( count($subcategories) && !$visible_subcats ) {
					$visible_cats--;
				}
			}
		}
		if( count($categories) && !$visible_cats ) {
			return false;
		}
		
		// Match start date and sold out flag
		if( ($event->get_months_until_start() > $this->params['num_months']) ||
			($event->is_today() && !$this->params['show_today']) ||
			($event->is_sold_out() && !$this->params['show_sold_out'])
		) {
			// We can't match the required parameters
			return false;
		}
		
		// If showing multiple currencies,
		if( $this->params['currency_list'] ) {
			foreach( $this->params['currency_list'] as $currency => $processor ) {
				if( $event->get_price_by_currency( $currency ) ) {
					// Price found in one of the currencies
					return true;
				}
			}
			
		}
		// If showing single currency,
		else if( $event->get_price_by_currency($this->params['currency']) ) {
			// Price found in the global currency
			return true;
		}
		
		// We can't find matching price
		return false;
	}
	
	public function event_data($event) {
		
		$dates = $event->get_dates($this->params['show_classroom_dates']);
		$times = $event->get_times($this->params['show_classroom_dates']);

		$months_until_start = $event->get_months_until_start();
		
		// Tag table row with categories
		//
		$cats    = array();
		$subcats = array();
		
		foreach( $event->get_categories() as $category ) {
			array_push($cats, $category->get_id());
			foreach( $category->get_subcategories() as $subcategory ) {
				array_push($subcats, $subcategory->get_id());
			}
		}
		
		$categories_str    = join(',', $cats);
		$subcategories_str = join(',', $subcats);
		
		// If multi-currency table 
		// use the event default currency for each event
		//
		if( is_array($this->params['currency_list']) ) {
			
			$event_currency = $event->get_default_currency();
			
			if( !array_key_exists($event_currency, $this->params['currency_list']) && 
				$event->get_price_by_currency( $this->params['currency'] )
			){
				$event_currency = $this->params['currency'];
			}
		}
		// If single-currency table 
		// use the table currency for all events
		//
		else {
			$event_currency = $this->params['currency'];
		}
		
		// Tag table row with currencies
		//
		if( is_array($this->params['currency_list']) ) {
			
			if( $event->get_price_by_currency( $event_currency )->is_zero() ) {
				// Tag free events with all currencies
				$event_currency_list = array_keys($this->params['currency_list']);
			}
			else{
				// Tag event with matchign currencies
				$event_currency_list = array();
				
				foreach( $this->params['currency_list'] as $currency_name => $processor) {
					$currency_price = $event->get_price_by_currency($currency_name);
					
					if( !$currency_price ) {
						continue;
					}
					$event_currency_list[] = $currency_name;
				}
			}
		}
		else {
			// Single-currency
			$event_currency_list = array( $this->params['currency'] );
		}
		$currencies_str = join(',', $event_currency_list);
		
		// Return event tags
		return array(
			"categories_str"      => $categories_str,
			"subcategories_str"   => $subcategories_str,
			"currencies_str"      => $currencies_str,
			"months_until_start"  => $months_until_start,
			"dates"               => $dates,
			"times"               => $times,
			"cats"                => $cats,
			"subcats"             => $subcats,
			"event_currency"      => $event_currency,
			"event_currency_list" => $event_currency_list,
		);
	}
}
