<?php
// Base class for event Table, Slider and List
//
abstract class AdministrateLearningTrackWidget extends AdministrateWidget2 {

	public function run($widget_params = array()) {
		$this->configure($widget_params);
	}

	public function configure($widget_params) {
		$global_params = array(
			'group_size' => $this->service->get_int_option('group_size', 'learning_tracks'),
			'no_learning_tracks'=> $this->service->get_option('no_learning_tracks', 'learning_tracks', 'error_message'),
		);

		if(function_exists("shortcode_atts")) {
			$this->params = shortcode_atts($global_params, $widget_params);
		} else {
			$this->params = $global_params;
		}
	}

	public static function get_price($track, $currency = null) {

		// track must have prices, and currency must be specified
		if(count($track->prices) == 0 || is_null($currency)) {
			return null;
		}

		// NOTE: array_filter does not reset the indexes after it runs, if the zero'th element is removed
		// by the filter, then [0] will return NULL later. Hence why we use reset($prices) below.
		$prices = array_filter($track->prices, function($price) use($currency) {
			return $price->currency == $currency;
		});

		if(count($prices) == 0) {
			return null;
		}

		$best_price = reset($prices);

		foreach($prices as $price) {
			if($price->currency == $currency && $price->amount < $best_price->amount) {
				$best_price = $price;
			}
		}

		return $best_price;
	}

	public static function display_price($track, $currency, $symbol, $basis) {
		$price = self::get_price($track, $currency);
		if(is_null($price)) {
			return array('<em>No Price Specified</em>', null);
		}

		$return_str = $symbol;
		if(property_exists($price, 'tax_type') && !is_null($price->tax_type)) {
			if ($basis == 'gross') {
				$prices_include_taxes = true;
				$return_str .= number_format($price->gross_amount, 2);
				$return_str .= ' <span class="administrate_tax_label"> <abbr title="including">inc.</abbr> <span>' . $price->tax_type->name . '</span> </span> ';
			} else {
				$prices_include_taxes = false;
				$return_str .= number_format($price->amount, 2);
				$return_str .= ' <span class="administrate_tax_label"> <abbr title="excluding">exc.</abbr> <span>' . $price->tax_type->name . '</span> </span> ';
			}
		} else {
			$prices_include_taxes = false;
			$return_str .= number_format($price->amount, 2);
		}

		return array($return_str, $prices_include_taxes);
	}

	public static function price_detail($track, $currency, $num_attendees, $discount_type=null, $discount_rate=null) {
		$price = self::get_price($track, $currency);

		if(!is_null($price->tax_type)) {
			$tax_name = $price->tax_type->name;
			$tax_rate = $price->tax_type->rate / 100;
		} else {
			$tax_name = '-';
			$tax_rate = 0;
		}

		$net_price = $price->amount * $num_attendees;
		$gross_price = $price->gross_amount * $num_attendees;

		if(!is_null($discount_type) && !is_null($discount_rate)) {
			if($discount_type == 'fixed') {
				$discount_amount = max(
					0,
					min(
						$discount_rate,
						$net_price
					)
				);

				$discounted_net_price = $net_price - $discount_amount;
				$tax_amount = $tax_rate * $discounted_net_price;
				$discounted_gross_price = $discounted_net_price + $tax_amount;
			} else {
				$discount_amount = $net_price * $discount_rate * $num_attendees;

				$discounted_net_price = $net_price - $discount_amount;
				$tax_amount = $tax_rate * $discounted_net_price;
				$discounted_gross_price = $discounted_net_price + $tax_amount;
			}
		} else {
			$tax_amount = $net_price * $tax_rate;
			$discounted_net_price = $net_price;
			$discounted_gross_price = $gross_price;
		}

		return array(
			$tax_amount,
			$tax_name,
			$net_price,
			$discounted_net_price,
			$gross_price,
			$discounted_gross_price,
		);
	}
}
