<?php
// Test class AdministrateAPIObject
//
use PHPUnit\Framework\TestCase;
require_once('common/api/objects/AdministrateAPIObject.php');
date_default_timezone_set("UTC");

class AdministrateAPIObjectTest extends TestCase {
	
	public function test_prepare_api_fields() {
		$test_fields = array(
			"id" => 10,
			"field1" => "first value",
			"field3" => "I",
			"field4" => time(),
			"unknown_key" => 20
		);
		
		$api_fields = AdministrateAPIObject_TestMock::prepare_api_fields($test_fields);
		
		$this->assertTrue( is_array($api_fields) );
		$this->assertEquals( 5, count($api_fields) );
		
		$this->assertArrayHasKey( "Test_ID", $api_fields );
		$this->assertEquals( 10, $api_fields["Test_ID"] );
		
		$this->assertArrayHasKey( "Field_1", $api_fields );
		$this->assertEquals( "first value", $api_fields["Field_1"] );
		
		$this->assertArrayHasKey( "Field_2", $api_fields );
		$this->assertSame( null, $api_fields["Field_2"]);
		
		$this->assertArrayHasKey( "Field_3", $api_fields );
		$this->assertEquals( "invoice", $api_fields["Field_3"]  );
		
		$this->assertArrayHasKey( "Field_4", $api_fields );
		$this->assertEquals( date('d m y / h:i', time()), $api_fields["Field_4"]  );
		
		$this->assertArrayNotHasKey( "unknown_key", $api_fields );
	}
		
	/**
	* @depends test_prepare_api_fields
	*/
	public function test_prepare_api_object() {
		$test_fields = array(
			"id" => 90000,
			"field1" => "Value",
			"field3" => "C",
			"field4" => time(),
			"unknown_key" => 20
		);
		
		$api_obj = AdministrateAPIObject_TestMock::prepare_api_object($test_fields);
		
		$this->assertTrue( is_object($api_obj) );
		$this->assertObjectHasAttribute("Test_ID", $api_obj);
		$this->assertEquals( 90000, $api_obj->Test_ID );
		
		$this->assertObjectHasAttribute("Field_1", $api_obj);
		$this->assertEquals( "Value", $api_obj->Field_1 );
		
		$this->assertObjectHasAttribute("Field_2", $api_obj);
		$this->assertSame( null, $api_obj->Field_2 );
		
		$this->assertObjectHasAttribute("Field_3", $api_obj);
		$this->assertEquals( "card", $api_obj->Field_3  );
		
		$this->assertObjectHasAttribute("Field_4", $api_obj);
		$this->assertEquals( date('d m y / h:i', time()), $api_obj->Field_4 );
		
		$this->assertObjectNotHasAttribute("unknown_key", $api_obj);
	}

	/**
	* @depends test_prepare_api_object
	*/
	public function test_get_id() {
		$test_fields = array(
			"id" => 101
		);
		$prep_obj = AdministrateAPIObject_TestMock::prepare_api_object($test_fields);
		$obj = new AdministrateAPIObject_TestMock($prep_obj);
		$id = $obj->get_id();
		
		$this->assertEquals( 101, $id );
	}
	
	/**
	* @depends test_prepare_api_object
	*/
	public function test_exists() {
		$test_fields = array(
			"id" => 101
		);
		$prep_obj = AdministrateAPIObject_TestMock::prepare_api_object($test_fields);
		$obj = new AdministrateAPIObject_TestMock($prep_obj);
		$this->assertTrue( $obj->exists() );
		
		$test_fields2 = array(
			"id" => 0
		);
		$prep_obj2 = AdministrateAPIObject_TestMock::prepare_api_object($test_fields);
		$obj2 = new AdministrateAPIObject_TestMock($prep_obj);
		$this->assertTrue( $obj2->exists() );
		
		$obj3 = new AdministrateAPIObject_TestMock();
		$this->assertFalse( $obj3->exists() );
	}
}

class AdministrateAPIObject_TestMock extends AdministrateAPIObject {
	private static $test_map = array(
		'id'	=>	array(
			'api_field'	=>	'Test_ID'
		),
		'field1'	=>	array(
			'api_field'	=>	'Field_1'
		),
		'field2'	=>	array(
			'api_field'	=>	'Field_2',
			'auto_fill' => true,
			'default' => null
		),
		'field3'	=>	array(
			'api_field'	=>	'Field_3',
			'mappings' => array(
				'I'	=>	'invoice',
				'C'	=>	'card'
			)
		),
		'field4'	=>	array(
			'api_field'	=>	'Field_4',
			'time_format' => 'd m y / h:i'
		),
	);
	protected static function _get_api_map() {
		return self::$test_map;
	}
}
