<?php
//  Include the widget abstract class
require_once('widgets/AdministrateWidget.php');

//  Include the price widget
require_once('widgets/event_price/AdministrateWidgetEventPrice.php');

//  Administrate Web Link Controller
class AdministrateWebLinkController {
	
	//  Constructor
	public function __construct(&$service, $action = false, $params = array()) {
		
		//  Start the session
		if(session_id() == '') {
			@session_start();
		}
		
		//  Save a reference to the service
		$this->service = &$service;
		
		//  Initialize the price widget
		$this->priceWidget = new AdministrateWidgetEventPrice($this->service);
	}
	
	//  Get widget class
	private function _get_widget_class($key) {
		$key = str_replace(' ', '', ucwords(str_replace('_', ' ', $key)));
		return 'AdministrateWidget' . $key;
	}
	
	//  Run a widget
	public function run($key, $params = array()) {
		
		//  Get the widget class name
		$widgetClass = $this->_get_widget_class($key);
		
		// Widgets common to Weblink and Wordpress
		if( in_array(
				$widgetClass,
				array(
					// Event Widgets
					"AdministrateWidgetEventTable", "AdministrateWidgetEventList", "AdministrateWidgetEventSlider",

					// LT Widgets
					"AdministrateWidgetLearningTrack", "AdministrateWidgetLearningTrackList"
				)
		)) {	
			$this->widget = new $widgetClass($this->service);
			return $this->widget->run($params);
		}
		// All other widgets
		else {
			require_once("widgets/" . $key . '/' . $widgetClass . '.php');
			$this->widget = new $widgetClass($this->service, $params);
			return $this->widget->run();
		}
	}
	
	//  Display the price widget widget
	public function display_price_widget(&$event, $fieldName, $defaultCurrency = '', $showCurrencyIndicator = true, $currencyIndicator = 'symbol') {
		echo $this->priceWidget->get_prices($event, $fieldName, $defaultCurrency, $showCurrencyIndicator, $currencyIndicator); 
	}
	
	//  Display the currency selector
	public function display_currency_selector(&$events, $defaultCurrency = '', $fieldName = '') {
		$this->priceWidget->display_selector($events, $defaultCurrency, $fieldName);	
	}
	
	//  Display the currency prices
	public function display_currency_prices($event, $pricingBasis = false, $defaultCurrency = '', $showCurrencyIndicator = true, $currencyIndicator = 'symbol', $currencyList=false) {
		$this->priceWidget->display_prices($event, $pricingBasis, $defaultCurrency, $showCurrencyIndicator, $currencyIndicator, $currencyList);	
	}
	
	//  Get a currency symbol
	public function get_currency_symbol($currency) {
		return $this->priceWidget->get_currency_symbol($currency);	
	}
	
	//  Format a currency
	public function format_currency($amount, $currency, $showCurrencyIndicator = true, $currencyIndicator = 'symbol') {
		return $this->priceWidget->format_currency($amount, $currency, $showCurrencyIndicator, $currencyIndicator);	
	}
	
}
