'use strict';

// Flag indicating if the browser had made a change to the # fragment
var browser_changed_hash = true;

// Valid widget names
var administrate_widget_names = [
    'category',
    'category_list',
    'checkout',
    'course',
    'course_list',
    'course_menu',
    'event',
    'event_list',
    'event_menu',
    'event_price',
    'event_slider',
    'event_table',
    'subcategory',
    'subcategory_list',
    'learning_track',
    'learning_track_list'
];

// Update the # fragment of the url
function set_or_replace_hash(target_id, widget_name, widget_params) {
        // Find current url hash for widgets
        var url_parts = window.location.href.split('#');
        if (url_parts.length == 1) {
            // no hash, create one
            var hash = '#' + target_id + '=' + widget_name + widget_params;
        } else {
            // hash exists, replace relevant widget part
            var hash = sanitize_hash( url_parts[1] );
            var hash_parts = hash.split('|');
            var found = false;
            for (var i=0; i<hash_parts.length; i++) {
                if (hash_parts[i].indexOf(target_id + '=') == 0) {
                    hash_parts[i] = target_id + '=' + widget_name + widget_params;
                    var found = true;
                    break;
                }
            }
            if (!found) {
                hash_parts.push(target_id + '=' + widget_name + widget_params);
            }
            var hash = '#' + hash_parts.join('|');
        }
        //set hash but dont refresh
        browser_changed_hash = false;
        window.location.hash = hash;
}

// Sanitize # fragment of the url
//
// The fragment has to be correctly formated or the active widget
// would not be recognized.
//
// Weblink expects the url to be in the format:
// <base-url>#widget=name?params
//
function sanitize_hash(hash) {
    var hash_parts = hash.split("?");
    
    // The widget=name part of the url has to be decoded,
    // as SagePay will encode that part.
    //
    // The parameters should not be decoded, because they may
    // contain course codes with special characters.
    //
    var widget_str        = decodeURIComponent(hash_parts[0]);
    var widget_params_str = hash_parts.length < 2 ? "" : "?"+hash_parts[1];
    
    // Find the longest matching widget name
    var clean_str_longest = '';

    for(var i=0; i < administrate_widget_names.length; i++) {
        var clean_str = "widget="+administrate_widget_names[i];
        
        if( widget_str.indexOf(clean_str) != -1) {
            if(clean_str.length > clean_str_longest.length) {
                clean_str_longest = clean_str;
            }
        }
    }

    if( clean_str_longest.length ) {
        widget_str = clean_str_longest;
    }

    return widget_str + widget_params_str;
}

// WORKAROUND FOR CORS ON IE <9.
// http://viejs.org/docs/2.0.0alpha1/xdr.html
// Override jQuery's default ajax transport layer to use XDomainRequest
var root = this;
(function( jQuery ) {

    // XDomainRequest only exists on IE <= 10, but only run it on <= 9 because IE10 supports native XMLHttpRequest
    if ( root.XDomainRequest && navigator.appVersion.indexOf("MSIE 10") == -1 ) {
        jQuery.ajaxTransport(function( s ) {
            if ( s.crossDomain && s.async ) {
                if ( s.timeout ) {
                    s.xdrTimeout = s.timeout;
                    delete s.timeout;
                }
                var xdr;
                return {
                    send: function( _, complete ) {
                        function callback( status, statusText, responses, responseHeaders ) {
                            xdr.onload = xdr.onerror = xdr.ontimeout = jQuery.noop;
                            xdr = undefined;
                            complete( status, statusText, responses, responseHeaders );
                        }
                        xdr = new XDomainRequest();
                        if(s.dataType){
                            var headerThroughUriParameters = "header_Accept=" + encodeURIComponent(s.dataType);
                            s.url = s.url + (s.url.indexOf("?") === -1 ? "?" : "&" ) + headerThroughUriParameters;
                        }
                        xdr.open( s.type, s.url );
                        xdr.onload = function(e1, e2) {
                            callback( 200, "OK", { text: xdr.responseText }, "Content-Type: " + xdr.contentType );
                        };
                        xdr.onerror = function(e) {
                            console.error(JSON.stringify(e));
                            callback( 404, "Not Found" );
                        };
                        if ( s.xdrTimeout ) {
                            xdr.ontimeout = function() {
                                callback( 0, "timeout" );
                            };
                            xdr.timeout = s.xdrTimeout;
                        }
                        xdr.send( ( s.hasContent && s.data ) || null );
                    },
                    abort: function() {
                        if ( xdr ) {
                            xdr.onerror = jQuery.noop();
                            xdr.abort();
                        }
                    }
                };
            }
        });
    }
})( jQuery );

// Create closure.
(function($) {

    // Plugin definition.
    $.fn.administrate = function(key, options) {

		// If the first argument was actually the options, copy the key to the options
		if ($.type(key) != 'string') {
			options = key;
		} else if (!options) {
			options = {};
		}

		// Extend our default options with those provided.
		var options = $.extend({}, $.fn.administrate.defaults, administrate, options);

		// Set the service URL
		options.serviceUrl = options.clientUrl.substr(0, options.clientUrl.length-11);

		//  Set the widget class
		options.widgetClass = options.namespace + '-' + options.widgetClass;

		//  Set the spinner class
		options.spinnerClass = options.namespace + '-' + options.spinnerClass;

		// If a widget key was passed in, run just the requested widget
		if ($.type(key) == 'string') {

            // Find the widget container id
            var target_id = this.context.id;

            // Find the target widget name
            var part = key.substr(get_service_url().length);
            part = part.substr(9);  // remove '/widgets/'

            var key_parts = part.split('?');

            //remove trailing slash
            if (key_parts[0].substr(key_parts[0].length-1) == '/') {
                var widget_name = key_parts[0].slice(0,-1);
            } else {
                var widget_name = key_parts[0];
            }

            var widget_params = '';
            // get params specific to target widget
            if (widget_name == 'checkout') {
                var url_options = key_parts[1].split('&');
                for (var i=0; i<url_options.length; i++) {
                    if (url_options[i].indexOf('event_id=') == 0) {
                        widget_params = '?' + url_options[i];
                        break;
                    } else if (url_options[i].indexOf('track_id=') == 0) {
                        widget_params = '?' + url_options[i];
                        break;
                    }
                }
                
                // Checkout widget is called form another widget,
                // transfer the relevant options.
                var opt_list = [
                    'currency',
                    'event_currency',
                    'payment_by_invoice',
                    'payment_by_cc',
                    'payment_processor',
                    'paypal_mode',
                    'sagepay_mode',
                    'authorizenet_mode',
                    'sagepay_email_message',
                    'show_promo_codes',
                    'force_email_check',
                    'show_classroom_dates',
                    'objectives'
                ];
                
                $.each( url_options, function(index,opt) {
                    var opt_parts = opt.split("=");
                    
                    if ( $.inArray(opt_parts[0], opt_list ) != -1 ) {
                        options[ opt_parts[0] ] = opt_parts[1];
                    }
                });
                
                // If a currency filter is active, checkout in the filter currency 
                // instead of the event default currency.
                //
                var currency_filter = $('#administrate-currency-select').val();
                
                if( currency_filter && (currency_filter!="all") ) {
                    options['event_currency'] = currency_filter;
                }
                
            } else if (widget_name == 'course') {
                var url_options = key_parts[1].split('&');
                for (var i=0; i<url_options.length; i++) {
                    if (url_options[i].indexOf('course_code=') == 0) {
                        widget_params = '?' + url_options[i];
                        break;
                    }
                }
            } else if (widget_name == 'course_list') {
                //clear out settings from previous pages
                widget_params = '?category_id=0&subcategory_id=0';
            } else if (widget_name == 'category') {
                var url_options = key_parts[1].split('&');
                for (var i=0; i<url_options.length; i++) {
                    if (url_options[i].indexOf('category_id=') == 0) {
                        widget_params = '?' + url_options[i];
                        break;
                    }
                }
            } else if (widget_name == 'subcategory') {
                var url_options = key_parts[1].split('&');
                var params = [];
                for (var i=0; i<url_options.length; i++) {
                    if (url_options[i].indexOf('subcategory_id=') == 0) {
                        params.push(url_options[i]);
                    }
                }
                widget_params = '?' + params.join('&');
            } else if (widget_name == 'subcategory_list') {
                var url_options = key_parts[1].split('&');
                for (var i=0; i<url_options.length; i++) {
                    if (url_options[i].indexOf('category_id=') == 0) {
                        widget_params = '?' + url_options[i];
                        break;
                    }
                }
            } else if (widget_name == 'learning_track') {
                var url_options = key_parts[1].split('&');
                for(var i=0; i<url_options.length; i++) {
                    if(url_options[i].indexOf('track_id=') == 0) {
                        widget_params = '?' + url_options[i];
                        break;
                    }
                }
            }

            set_or_replace_hash(target_id, widget_name, widget_params);

			run_widget(key, $(this), options);
			

		//  Otherwise this is the page initialization and we should find and initialize all widgets on the page
		} else {
			
			// Set the head and body
			var head = $(document.head);
			//var body = $(document.body);
			
			// Load the CSS
			include_css('weblink-components/resources/widgets-common.css');
			include_css('common/administrate-common.css');

			include_css('weblink-components/widgets/checkout/style.css');
			include_css('weblink-components/widgets/course/style.css');
			include_css('weblink-components/widgets/event_price/style.css');
			
			include_css('common/widgets/event_list/event_list.css');
			include_css('common/widgets/event_slider/event_slider.css');
			include_css('common/widgets/event_table/event_table.css');
			$.getScript( get_service_url('common/widgets/event_slider/event_slider.js') );
			$.getScript( get_service_url('common/widgets/event_list/event_list.js') );

			include_css('common/widgets/learning_track/learning_track.css');
			include_css('common/widgets/learning_track_list/learning_track_list.css');
            
			//  Loop through widgets and run them
			$.each(administrate_widget_names, function(i, key) {
				init_widgets(key);
			});
			
			//  Add event delegation for all links in widgets
			$('.' + options.widgetClass).on('click', function(e) {
 				
				//  jQuerify this
				var link = $(e.target)[0];
				
				//  Only proceed if this is a link
				// TODO: This event handler should hook to widget links, not everything in all widgets
				// Refactoring it now is pointless as it will be changed with the updated html/css.
				if ( (link.tagName == 'A') && (!$(link).hasClass('ui-tabs-anchor')) ) {

					//  Get the HREF
					var href = $(link).attr('href');

					//  If the HREF starts with the service URL, hijack it
					if (href.substr(0, get_service_url().length) == get_service_url()) {

						//  Run the appropriate widget
						if (href.indexOf('?') == -1) {
							href += '?';
						} else {
							href += '&';
						}

						$(this).administrate(href + $.param(options));

						//  Don't actually follow the link
						return false;
						
					}
				
				}
			
			});
		
		}

		// Init a widget
		function init_widgets(key) {

			//  Get the full data attribute
			var dataAttr = options.namespace + '-' + key.split('_').join('-');

			//  Get a reference to all widget instances
			var objs = $('[data-' + dataAttr +']');
			
			//  Loop over each instance
			$.each(objs, function(i, obj) {

				//  Parse out the widget options
				obj = $(obj);
				var tmpOptions = options;
				$.extend(tmpOptions, parse_widget_options(obj.data(dataAttr)));

				//  Run the widget
				run_widget(key, obj, tmpOptions);
				
			});

		}

		//  Run a widget
		function run_widget(key, obj, params) {
			
            //check url to see if should load something else in widget
            var url_parts = window.location.href.split('#');
            var hash = false;
            if (url_parts.length > 1) {
                hash = sanitize_hash( url_parts[1] );
                var hash_parts = hash.split('|');
            }

            if (hash) {
                var id = obj.context.id;
                for (var i=0; i<hash_parts.length; i++) {
  
                    //look for current widget id in hash
                    if (hash_parts[i].indexOf(id + '=') == 0) {
                        //want everything after first '='
                        var widget_string = hash_parts[i].substr(id.length + 1);
                        if (widget_string.length > 0) {
                            var settings = widget_string.split('?');
                            //set widget type (course, event list etc)
                            key = settings[0];
                            if (settings.length > 1) {
                                var setting_params = settings[1].split('&');
                                for (var j=0; j<setting_params.length; j++) {
                                    var tokens = setting_params[j].split('=');
                                    params[tokens[0]] = tokens[1];
                                }
                            }
                        }
                        break;
                    }
                }
            }

			// Add the widget class to the element
			obj.addClass(options.widgetClass);
      
			// If the key starts with 'http', it is the full URL including options
			if (key.substr(0,4) == 'http') {
				var url = key;
				params = {};
			
			//  Otherwise generate the URL and assume passed params are good
			} else {
				var url = get_service_url('widgets/'+key);
			}
			
			//  Create the loader DIV
			var spinner = $('<div class="' + options.spinnerClass + '"></div>');
			obj.append(spinner);
			
			//  Make the request
			$.get(
				url,
				params,
				function(data, status) {
					
					// Inject HTML
					obj.html(data);
					
					// Initialize widgets
					var prefix = 'administrate-';
					var checkoutWidgetId = prefix+'checkout';
					var eventTableWidgetId = prefix+'event-table';
					var eventSliderWidgetClass = prefix+'event-slider2';
					var eventListWidgetClass = prefix+'event-list';
					var currencySelectorWidgetClass = prefix+'event-price-currencies';
					
					//  Add 'js' class to body
					$(document.body).addClass(prefix+'js');
					
					//  Initialize the checkout widget if it exists
					var checkoutWidget = $('#'+checkoutWidgetId);
					if (checkoutWidget) {
						checkoutWidget.administrate_checkout();
					}
					
					//  Initialize the event table widget if it exists
					var eventTableWidget = $('#'+eventTableWidgetId);
					if (eventTableWidget) {
						eventTableWidget.administrate_event_table();	
					}
					
					//  Initialize the event table widget if it exists
					var eventSliderWidgets = $('.'+eventSliderWidgetClass);
					$.each(eventSliderWidgets, function(i, slider) {
						administrate_event_slider($, slider);
					});
                    
                    var eventListWidgets = $('.'+eventListWidgetClass);
                    $.each(eventListWidgets, function(i, slider) {
                        administrate_event_list($, slider);
                    });
					
					//  Initialize the currency selector widgets if present
					var currencySelectorWidgets = $('.'+currencySelectorWidgetClass);
					$.each(currencySelectorWidgets, function(i, el) {
						$(el).administrate_event_price();
					});
				}
				
			);
		
		};

	    //  Parse widget options
	    function parse_widget_options(str, key) {

	    	//return $.parseJSON(str.replace(/\'/g, '"'));
	    	//return eval(str);

	    	//  Initialize options object
	    	var options = {};

	    	//  Loop through options
	    	$.each(str.split('|'), function(i, opt) {

	    		// Split by equals sign
	    		var parts = opt.split('=');

	    		//  First part is the key
	    		var key = $.trim(parts[0]);

	    		//  Second part is the value
	    		var val = $.trim(parts[1]);

	    		//  If the value is a comma-delimited list, parse it as an array
	    		if (val.indexOf(',') > 0) {
	    			val = val.split(',');
	    		}

	    		//  Set the option
	    		options[key] = val;

	    	});

	    	return options;

	    };

        // Include CSS
	    function include_css(path) {
	        var css = $('<link rel="stylesheet" type="text/css" href="' + get_service_url(path) + '">');
	        css.appendTo('head');
	    };

	    // Get client URL
	    function get_url(path) {
	    	if (!path) {
				path = '';	
			} else {
				path = '/' + path;	
			}
			return options.clientUrl + path;
	    };
		
		// Get service URL
	    function get_service_url(path) {
	    	if (!path) {
				path = '';	
			} else {
				path = '/' + path;	
			}
			return options.serviceUrl + path;
	    };

    	// Log
		function log(msg) {
			if (options.debug && window.console && window.console.log) {
				console.log(msg);
			}
		};

    };

    // Plugin defaults – added as a property on our plugin function.
	$.fn.administrate.defaults = {
	    namespace:		'administrate',
	    debug:			false,
	    clientUrl:		administrate.clientUrl,
		widgetClass:	'widget',
		spinnerClass:	'spinner'
	};

	//  Initialize the plugin
	$(document).ready(function() {
		$(document.documentElement).administrate(administrate);
        window.onhashchange = function () {
            //refresh page if hash change not due to widget link click
            //i.e. back/forward button
            if (browser_changed_hash) {
                window.location.reload();
            } else {
                browser_changed_hash = true;
            }
        }
	});

})(jQuery);
