<?php
//  Widget parent class
abstract class AdministrateWidget {

	//  Properties
	protected $key = '';
	protected $shortCode = true;
	protected $showOnce = false;
	protected $alreadyShown = false;
	protected $widgetPath = '';
	protected $service;

	/* @var $params array */
	protected $params = array();

	//  Constructor
	public function __construct(&$service, $params = array()) {
	
		//  Save the service by reference
		$this->service = &$service;
		
		//  Save the params
		$this->params = $params;
		
		//  Set the widget path
		$reflector = new ReflectionClass(get_class($this));
		$this->widgetPath = dirname($reflector->getFileName());
		$this->widgetUrl = $this->service->get_url('/controllers/widgets/' . $this->key);
	
	}
	
	//  Run the widget code
	public function run() {
		$this->alreadyShown = true;
	}
	
	//  Get the parsed content of a file include
	protected function get_include_contents($filename, $params = array()) {
    	if (is_file($filename)) {
			ob_start();
        	include($filename);
        	$contents = ob_get_contents();
        	ob_end_clean();
        	return $contents;
    	} else {
    		return false;
		}
	}
	
	//  Get the widget path
	public function get_path($path) {
		return $this->widgetPath.$path;	
	}
	
	//  Get the widget URL
	protected function _get_url($path) {
		return $this->widgetUrl.$path;	
	}
	
	//  Add a namespace to the passed arguments
	public function add_namespace($parts, $delimiter = false) {
		if (empty($parts)) {
			$parts = array();
		} else if (!is_array($parts)) {
			$parts = array($parts);
		}
		array_unshift($parts, $this->key);	
		return $this->service->add_namespace($parts, $delimiter);
	}
	
	//  Strip out the namespace from a string
	public function strip_namespace($str, $delimiter = false) {
		return substr($str, strlen($this->add_namespace('', $delimiter)) + 1);	
	}
	
	//  Whether or not a parameter is truthy
	protected function _is_true($var) {
		return (($var === true) || ($var === 'true') || (intval($var) === 1));	
	}
	
	//  Convert a parameter to an array
	protected function _convert_to_array($var) {
		if (!is_array($var)) {
			$var = array_map('trim', explode(',', $var));	
		}
		return $var;
	}
	
	//  Whether or not the page is generated
	public function page_is_generated() {
		return true;	
	}

}
