<?php
//  Include required classes
if(!class_exists('AdministrateDao')) {
	require_once('AdministrateDao.php');
}

require_once(__DIR__ .'/../../../common/api/MappedItemCalculator.php');

//  Administrate checkout widget order
class AdministrateWidgetCheckoutOrder extends AdministrateDao {
	
	//  Properties
	protected $tableKey = 'orders';
	protected $primaryKey = 'order_id';
	protected $apiKeyField = 'order_api_key';

	/* @var $eventPrice AdministrateAPIEventPrice */
	protected $eventPrice = null;

	private $track = null;
	private $course = null;

	//  Create a new order
	public function create($event_id = array(), $session_id = false, $currency = false, $track_id = null) {
		if (!$currency) {
			$currency = $this->service->get_option('currency');
		}

		if(is_null($track_id)) {
			return parent::create(array(
				'order_session_id' => $session_id,
				'order_event_id' => $event_id,
				'order_currency' => $currency,
				'order_time_started' => time()
			));
		} else {
			return parent::create(array(
				'order_session_id' => $session_id,
				'order_track_id' => $track_id,
				'order_currency' => $currency,
				'order_time_started' => time()
			));
		}
	}
	
	public function receive_ipn() {

		require_once($this->get_path('/AdministrateWidgetCheckoutLog.php'));
		$log = new AdministrateWidgetCheckoutLog($this->service);
		$log->create('Running $order->receive_ipn();', $this->get_id());

		// If we don't have an invoice ID, then the user hasn't yet
		// visited step 5 - so simply process the order.
		$orderNum = $this->get_api_invoice_id();
		if(!$orderNum) {

			$log->create('No api_invoice_id set yet, submitting order.', $this->get_id());
			$orderNum = $this->complete(true);
			$log->create('Got $orderNum: ' . $orderNum . '.', $this->get_id());
		} else {
			$log->create('api_invoice_id already set - nothing further to do here.', $this->get_id());
		}

		// DISABLED WHILST PENDING ORDERS ARE DISABLED - NO NEED TO COMPLETE COMPLETE THINGS.
		// Tell Administrate that the order is now complete
		//try {
		//	$log->create('Attempting to mark order as completed.', $this->get_id());
		//	Administrate::api()->raw_call('completeOrder', $this->get_api_invoice_id());
		//	$log->create('Order marked as completed.', $this->get_id());
		//}
		//catch(Exception $e) {
		//	$log->create('Couldn\'t mark order as completed: ' . $e->getMessage(), $this->get_id());
		//	return false;
		//}

		// And ensure we update the status of the order in the plugin DB.
		$fields = array(
			'order_time_completed'				=>	time(),
			'order_status'						=>	'C',
			'order_payment_type'				=>	'C',
			'order_api_invoice_id'				=>	intval($this->get_api_invoice_id())
		);

		// Save the order
		$this->save($fields);

		$this->service->db->query('UPDATE wl_administrate_orders SET order_being_processed = 0 WHERE order_id = ' . $this->get_id() . ';');

	}

	//  Complete the order
	public function complete($cc_straight_to_placed = true, $alt_addresses=array(), $use_sales_opportunities = false, $api_key = null) {

		require_once($this->get_path('/AdministrateWidgetCheckoutLog.php'));
		$log = new AdministrateWidgetCheckoutLog($this->service);
		$log->create('Running $order->complete(' . ($cc_straight_to_placed ? 'true' : 'false') . ');', $this->get_id());

		$log->create('Order payment type selected: ' . ((array_key_exists('order_payment_type', $_REQUEST) && $_REQUEST['order_payment_type'] == 'C') ? 'payment processor' : 'invoice') . '.', $this->get_id());
		$log->create('Currency order status: ' . $this->get_status(), $this->get_id());

		// Lock tables to mark order as being processed, so that we don't
		// process two orders at the same time.
		// By default, presume we cannot process.
		$can_process = 0;
		try {
			$this->service->db->query('LOCK TABLE wl_administrate_orders WRITE');

			$processed = $this->service->db->query_fetch(
				'SELECT order_being_processed FROM wl_administrate_orders WHERE order_id = ' . $this->get_id() . ';',
				'ARRAY_A'
			);

			// Order is not being processed, so let's mark that we can.
			if($processed['order_being_processed'] == 0) {
				$this->service->db->query('UPDATE wl_administrate_orders SET order_being_processed = 1 WHERE order_id = ' . $this->get_id() . ';');
				$can_process = 1;
			}

			$this->service->db->query('UNLOCK TABLES;');
		} catch (Exception $e) {

			$this->service->db->query('UNLOCK TABLES;');
			throw $e;
		}

		//  Only process if the order is already complete
		if (!$this->is_complete() && $can_process == 1) {

			$log->create('Order not currently \'completed\' or being processed - proceeding to process.', $this->get_id());

			//  Initialize the API
			$this->service->init_api($api_key);

			// Default paymentType to 'C', and if we're paying by invoice then update accordingly.
			$paymentType = 'C';
			if(Administrate::get_is_invoice_payment_from_post()) {
				$paymentType = 'I';
			}

			if(!($this->service->to_boolean($this->service->get_option('use_sales_opportunities')) || $use_sales_opportunities)) {

				//  Try to get the user with email address
				$user = Administrate::api()->get_user_by_email($this->get_buyer_email(), false);

				//  If the user exists, get the existing delegates and initialize objects
				if (!$user->exists()) {
					$fields = array(
						'is_individual' => (strlen(trim($this->get_buyer_company())) == 0),
						'email' => $this->get_buyer_email(),
						'no_contact' => true,
						'company' => $this->get_buyer_company(),
						'first_name' => $this->get_buyer_first_name(),
						'last_name' => $this->get_buyer_last_name(),
						'phone' => $this->get_buyer_phone(),
						'address1' => $this->get_invoice_street(),
						'city' => $this->get_invoice_city(),
						'territory' => $this->get_invoice_territory(),
						'postal_code' => $this->get_invoice_postal_code(),
						'country' => $this->get_invoice_country()
					);
					$user = Administrate::api()->add_user($fields);
				}

				//  Loop through attendees and add any that aren't already delegates
				$attendees = $this->get_attendees();
				foreach ($attendees as $i => $attendee) {
					$delegate = $user->get_delegate_by_email($attendees[$i]['email']);

					if (!$delegate) {
						$delegate = Administrate::api()->add_delegate(array(
							'user_id' => $user->get_id(),
							'email' => $attendees[$i]['email'],
							'first_name' => $attendees[$i]['first_name'],
							'last_name' => $attendees[$i]['last_name'],
							'notes' => (array_key_exists('notes', $attendees[$i]) ? $attendees[$i]['notes'] : ''),
							'is_individual' => (strlen(trim($this->get_buyer_company())) == 0),
							'no_contact' => true,
							'company' => $this->get_buyer_company(),
							'phone' => $this->get_buyer_phone(),
							'address1' => $this->get_invoice_street(),
							'city' => $this->get_invoice_city(),
							'territory' => $this->get_invoice_territory(),
							'postal_code' => $this->get_invoice_postal_code(),
							'country' => $this->get_invoice_country(),
						));
					}

					if (is_object($delegate)) {
						$attendees[$i]['delegate_id'] = $delegate->get_id();
					}
				}

				//  Loop through attendees and add them as cart entries
				$cartEntries = array();
				foreach ($attendees as $attendee) {
					array_push(
						$cartEntries,
						array(
							'event_id' => $this->get_event_id(),
							'delegate_id' => $attendee['delegate_id']
						)
					);
				}

				/* Set the order details. We set is_pending if it's a credit
				 * card payment and we're not forcing it straight to placed.
				 */
				$orderDetails = array(
					// Not currently setting orders to pending because we're having too many failures from PayPal's IPN currently.
					'is_pending' => false, // ($paymentType == 'C' && !$cc_straight_to_placed),
					'date' => time(),
					'user_id' => $user->get_id(),
					'payment_method' => $paymentType,
					'currency' => $this->get_currency(),
					'region' => $this->get_event_region_id(),
					'email' => $this->get_buyer_email(),
					'company' => $this->get_buyer_company(),
					'first_name' => $this->get_buyer_first_name(),
					'last_name' => $this->get_buyer_last_name(),
					'phone' => $this->get_buyer_phone(),
					'address1' => $this->get_invoice_street(),
					'city' => $this->get_invoice_city(),
					'territory' => $this->get_invoice_territory(),
					'postal_code' => $this->get_invoice_postal_code(),
					'country' => $this->get_invoice_country()
				);
				if (!empty($alt_addresses)) {
					$orderDetails["alt_addresses"] = $alt_addresses;
				}
				if ($this->get_promo_code() != '') {
					$orderDetails['promo_code'] = $this->get_promo_code();
				}

				//  Create the order object the API
				$orderObj = array(
					'order_details' => $orderDetails,
					'cart_entries' => $cartEntries
				);

				// If this is not a PayPal IPN response, submit the order.
				if (!($paymentType == 'C' && $this->get_status() == 'P')) {

					$log->create('Submitting order to Administrate API.', $this->get_id());
					$orderNum = Administrate::api()->submit_order($orderObj);

					// Order can't be placed
					if (!$orderNum) {
						$this->service->db->query('UPDATE wl_administrate_orders SET order_being_processed = 0 WHERE order_id = ' . $this->get_id() . ';');
						$log->create('Didn\'t get an order number.', $this->get_id());
						return false;
					}
					$log->create('Got an order ID back - ' . $orderNum . '.', $this->get_id());
				}

				//  Send email the invoice
				$sendEmail = false;
				if ($paymentType == 'I') {
					$sendEmail = $this->service->to_boolean($this->service->get_option('invoice_send_email'));
					// If this is not a PayPal IPN response, submit the email.
				} elseif ($paymentType == 'C' && $this->get_status() == 'I') {
					$sendEmail = $this->service->to_boolean($this->service->get_option('cc_send_email'));
				}
				if ($sendEmail) {
					$log->create('Sending invoice email.', $this->get_id());
					$emailSent = Administrate::api()->send_invoice($orderNum);
				} else {
					$log->create('Settings dictate that no email should be sent.', $this->get_id());
				}
			} else {
				// Use the opps endpoints... YEY!
				$orderDetails = array(
					'buyer_information' => array(
						'first_name' => $this->get_buyer_first_name(),
						'last_name' => $this->get_buyer_last_name(),
						'email' => $this->get_buyer_email(),
						'company' => array(
							'name' => $this->get_buyer_company(),
							'address1' => $this->get_invoice_street(),
							'city' => $this->get_invoice_city(),
							'county' => $this->get_invoice_territory(),
							'postcode' => $this->get_invoice_postal_code(),
							'country' => $this->get_invoice_country()
						)
					),
					'cart_information' => array(),
					'currency_id' => $this->get_currency()
				);

				$attendees = $this->get_attendees();
				foreach ($attendees as $i => $attendee) {
					$del_info = array(
						'email' => $attendees[$i]['email'],
						'first_name' => $attendees[$i]['first_name'],
						'last_name' => $attendees[$i]['last_name'],
						'notes' => $attendees[$i]['notes']
					);

					if($this->get_event_id()) {
						$del_info['event_id'] = $this->get_event_id();
						$del_info['price'] = $this->get_event_net_price(false);
						$del_info['gross_price'] = $this->get_event_gross_price(false);
						$del_info['tax_type'] = $this->get_event()->get_tax_id();

						$del_info['children'] = array();
						if(count($this->get_item_ids())) {
							$items = $this->get_mapped_items_and_prices();
							foreach($items as $item) {
								$del_info['children'][] = array(
									'price' => $item['subtotal'],
									'item_id' => $item['id'],
									'is_discountable' => $item['is_discountable'],
									'is_taxable' => $item['is_taxable']
								);
							}
						}
					} else {
						$price = AdministrateLearningTrackWidget::get_price($this->get_track(), $this->get_currency());
						$del_info['track_id'] = $this->get_track_id();
						$del_info['price'] = $price->amount;
						$del_info['gross_price'] = $price->amount;
						if(!is_null($price->tax_type)) {
							$del_info['tax_type'] = $price->tax_type->id;
						}
					}

					$orderDetails['cart_information'][] = $del_info;
				}

				if($paymentType == 'C') {
					$orderDetails['payment_type'] = 'Card';
				} else {
					$orderDetails['payment_type'] = 'Cash';
				}

				if($this->get_promo_code()) {
					$orderDetails['promo_code'] = $this->get_promo_code();
				}

				if ($this->get_objectives()) {
					$orderDetails['objectives'] = $this->get_objectives();
				}

				$order = Administrate::api()->place_opps_order($orderDetails);
				$orderNum = $order->order_id;
			}

			// Mark the order as complete in the database
			$orderStatus = 'C';

			$fields = array(
				'order_time_completed'				=>	time(),
				'order_status'						=>	$orderStatus,
				'order_payment_type'				=>	$paymentType,
				'order_api_invoice_id'				=>	intval($orderNum),
				'order_being_processed'				=> 	0
			);

			//  Save the order
			$result = $this->save($fields);

			//  Return the order number
			return $orderNum;

		//  Or if the order is already complete, or is already processing
		} else {

			// Wait for up to 10 seconds to see if this order is already being
			// processed - we still want to return an order id if we can.
			if(!$this->is_complete() && $can_process == 0) {
				$log->create('Blocked from processing because processing is already in progress.', $this->get_id());
				$timeout = 1;
				$complete = 0;
				$completed = null;
				while(!$complete && $timeout <= 10) {
					$completed = $this->service->db->query_fetch(
						'SELECT order_status, order_api_invoice_id FROM wl_administrate_orders WHERE order_id = ' . $this->get_id() . ';'
					);
					$complete = ($completed['order_status'] != 'I');
					$log->create('Order is ' . ($complete ? '' : 'not') . ' complete after ' . $timeout . ' retries.', $this->get_id());
					$timeout++;
					sleep(1);
				}

				// If it's still being processed, log an error and return
				if(!$complete) {
					$log->create('Order did not complete after ' . $timeout . ' retries, returning false.', $this->get_id());
					return false;
				} else {
					return $completed['order_api_invoice_id'];
				}
			} else {
				$log->create('Order already complete.', $this->get_id());
			}

			return $this->get_api_invoice_id();
		}

	}

	//  Validate fields
	protected function _validate_fields($fields = array()) {
	
		//  Loop through field definitions
		$errors = array();
		$delegateEmails = array();
		$check_attendee_email = ($this->service->get_option('force_email_check') == 1);
		foreach ($this->fieldDefinitions as $field=>$properties) {
			if (isset($fields[$field])) {
				
				// If this field has serialized subvalues, check them
				if (isset($properties['serialize'])) {
					
					//  Loop through serialized fields and add an error for anything empty
					foreach ($properties['serialize'] as $key=>$subfield) {
						$subfieldKey = $field . $this->service->get_key_delimiter() . $subfield;
						if (is_array($fields[$subfieldKey])) {
							foreach ($fields[$subfieldKey] as $multiField) {
								$is_attendee_email_field = (($field == 'order_attendee_details') && ($subfield == 'email'));

								// Validation specific to the attendee email field
								if($is_attendee_email_field) {

									if(!$check_attendee_email && empty($multiField)) {

										// If we're not checking these, and it's empty, it's fine - just skip
										continue;
									} else {

										// If it has a value, we must ensure it's valid.
										if(!filter_var($multiField, FILTER_VALIDATE_EMAIL)) {
											$errors[$subfieldKey.$multiField] = $multiField . $this->service->get_text(' is not a valid email address.');
										}
									}
								}

								// If check if the field is empty, and throw validation warning
								if (empty($multiField)) {
									$errors[$subfieldKey] = $this->service->get_data_label($this->tableKey, $field, $subfield) . $this->service->get_text(' field is required.');
								}
								// If it's an attendee email field, check it's unique.
								if ($is_attendee_email_field) {
									if (in_array($multiField, $delegateEmails)) {
										$errors[$field] = $this->service->get_text('All attendees must have a unique email address.');
									} else {
										array_push($delegateEmails, $multiField);
									}
								}

							}
						} else {

							// Don't validate company, this is to accommodate 'individuals.'
							// Don't require territory - some regions/countries don't use this.
							if($subfield == 'company' || $subfield == 'territory') {
								continue;
							}

							// Check empty fields.
							if(empty($fields[$subfieldKey])) {
								$errors[$subfieldKey] = $this->service->get_data_label($this->tableKey, $field, $subfield) . $this->service->get_text(' field is required.');
							}

							// If this is an email field, ensure it is a valid email
							if($subfield == 'email') {
								if(!filter_var($fields[$subfieldKey], FILTER_VALIDATE_EMAIL)) {
									$errors[$subfieldKey] = $this->service->get_data_label($this->tableKey, $field, $subfield) . $this->service->get_text(' is not a valid email address.');
								}
							}
						}
					}
					
				// else just check the field
				} else {
					if (isset($properties['null']) && !$properties['null'] && empty($fields[$field])) {
						$errors[$field] = $this->service->get_data_label($this->tableKey, $field) . $this->service->get_text(' field is required.');
					}
				}
				
			}
		}
		
		//  Return the errors
		if (empty($errors)) {
			return false;	
		} else {
			return $errors;
		}
		
	}
	
	//  Get the max step
	public function get_max_step() {
		return $this->row['order_max_step'];	
	}
	
	public function get_tac() {
		return $this->row['tac'];	
	}
	
	// Get Promo Code
	public function get_promo_code() {
		return $this->row['order_discount'];
	}

	public function get_objectives() {
		if($this->row['order_path_objectives']) {
			return json_decode($this->row['order_path_objectives'], true);
		}
	}

	public function get_raw_objectives() {
		if($this->row['order_path_objectives']) {
			$json = json_decode($this->row['order_path_objectives'], true);
			$json_as_string = array();
			foreach($json as $chosen_objective) {
				$json_as_string[] = $chosen_objective['objective_id']."-".$chosen_objective['event_id'];
			}
			return implode(",", $json_as_string);
		}
	}

	//  Get the session ID
	public function get_session_id() {
		return $this->row['order_session_id'];	
	}
	
	//  Get the payment type
	public function get_payment_type() {
		return $this->row['order_payment_type'];	
	}
	
	//  Get the order status
	public function get_status() {
		return $this->row['order_status'];	
	}
	
	//  Whether or not the order is complete
	public function is_complete() {
		return ($this->get_status() == 'C');	
	}
	
	//  Get the order currency
	public function get_currency() {
		return $this->row['order_currency'];	
	}
	
	//  Get the time started
	public function get_time_started() {
		return $this->row['order_time_started'];	
	}
	
	//  Get the time completed
	public function get_time_completed() {
		return $this->row['order_time_completed'];	
	}
	
	public function get_is_being_processed() {
		return ($this->row['order_being_processed'] == 1);
	}

	/* **** BUYER DETAILS *** */
	
	//  Get the buyer details
	public function get_buyer_details() {
		return $this->row['order_buyer_details'];	
	}
	
	//  Get the buyer details fields
	public function get_buyer_details_fields() {
		$this->_set_buyer_details_fields();
		return $this->buyerDetails;	
	}
	
	//  Set buyer details
	private function _set_buyer_details_fields() {
		if (!property_exists($this, 'buyerDetails')) {
			$this->buyerDetails = unserialize(base64_decode($this->row['order_buyer_details']));
		}	
	}
	
	//  Get buyer field
	private function _get_buyer_field($field) {
		$this->_set_buyer_details_fields();
		return $this->buyerDetails[$field];
	}
	
	//  Get the buyer email
	public function get_buyer_email() {
		return $this->_get_buyer_field('email');
	}
	
	//  Get the buyer first name
	public function get_buyer_first_name() {
		return $this->_get_buyer_field('first_name');
	}
	
	//  Get the buyer last name
	public function get_buyer_last_name() {
		return $this->_get_buyer_field('last_name');
	}
	
	//  Get the buyer phone
	public function get_buyer_phone() {
		return $this->_get_buyer_field('phone');
	}
	
	//  Get the buyer compnay
	public function get_buyer_company() {
		return $this->_get_buyer_field('company');	
	}
	
	/* **** INVOICE ADDRESS *** */
	
	//  Get the invoice address
	public function get_invoice_address() {
		return $this->row['order_invoice_address'];	
	}
	
	//  Get the invoice address
	public function get_invoice_address_fields() {
		$this->_set_invoice_address_fields();
		return $this->invoiceAddress;	
	}
	
	//  Set invoice address
	private function _set_invoice_address_fields() {
		if (!property_exists($this, 'invoiceAddress')) {
			$this->invoiceAddress = unserialize(base64_decode($this->get_invoice_address()));
		}	
	}
	
	//  Get invoice field
	private function _get_invoice_field($field) {
		$this->_set_invoice_address_fields();
		return $this->invoiceAddress[$field];
	}
	
	//  Get the invoice address
	public function get_invoice_street() {
		return $this->_get_invoice_field('address');
	}
	
	//  Get the invoice city
	public function get_invoice_city() {
		return $this->_get_invoice_field('city');
	}
	
	//  Get the invoice territory
	public function get_invoice_territory() {
		return $this->_get_invoice_field('territory');
	}
	
	//  Get the invoice postal code
	public function get_invoice_postal_code() {
		return $this->_get_invoice_field('postal_code');
	}
	
	//  Get the invoice country
	public function get_invoice_country() {
		return $this->_get_invoice_field('country');
	}
	
	/* **** ATTENDEE DETAILS *** */
	
	//  Get the number of attendees
	public function get_num_attendees() {
		return $this->row['order_num_attendees'];	
	}

	public function get_item_ids() {
		return explode(',', $this->row['order_item_ids']);
	}

	public function set_item_ids($ids) {
		$fields = array(
			'order_item_ids'	=>	(count($ids) ? implode(',', $ids) : '')
		);
		$this->save($fields);
	}

	public function set_objective_ids($objectivesToSave) {
		$fields = array(
			'order_path_objectives'	=>	json_encode($objectivesToSave)
		);
		$this->save($fields);
	}

	//  Get the attendees
	public function get_attendee_details() {
		return $this->row['order_attendee_details'];	
	}

	public function get_tax_rate() {
		if($this->get_event_net_price(false) == 0) {
			return 0;
		}

		return $this->get_event_gross_price(false) / $this->get_event_net_price(false);
	}

	public function get_mapped_items_and_prices() {
		$calculator = new MappedItemCalculator();
		$items = $this->get_item_ids();
		$ret_array = array();
		foreach($items as $item_id) {
			$item = Administrate::api()->get_mapped_item($item_id);
			if(!is_object($item)) {
				continue;
			}

			$currency = $this->get_currency();

			// Can't derive prices if there are none.
			if(!is_array($item->item->prices)) {
				continue;
			}

			$prices = array_filter($item->item->prices, function($price) use($currency) {
				return $price->currency == $currency;
			});

			if(count($prices) == 0) {
				continue;
			}

			$best_price = reset($prices);

			foreach($prices as $price) {
				if($price->currency == $currency && $price->amount < $best_price->amount) {
					$best_price = $price;
				}
			}

			$item_data = array(
				'id' => $item->item->id,
				'name' => $item->item->name,
				'subtotal' => $best_price->amount * $this->get_num_attendees(),
				'discount' => (($best_price->is_discountable) ? round($best_price->amount * $this->get_discount_rate(), 2) * $this->get_num_attendees() : 0),
				'is_discountable' => $best_price->is_discountable,
				'is_taxable' => $best_price->is_taxable,
			);
			$item_data['tax'] = $calculator->calculate_tax($item_data, $this->get_tax_rate()) * $this->get_num_attendees();

			$ret_array[] = $item_data;
		}

		return $ret_array;
	}

	//  Get attendees
	public function get_attendees() {
		if (!property_exists($this, 'attendeeDetails')) {
			$tmp = unserialize(base64_decode($this->row['order_attendee_details']));
			$this->attendeeDetails = array();
			for ($i = 0, $numAttendees = count($tmp['first_name']); $i < $numAttendees; ++$i) {
				// E-mail must be valid
				if(!is_string($tmp['email'][$i])) {
					continue;
				}
				// Push attendee
				array_push(
					$this->attendeeDetails, 
					array(
						'first_name'	=>	$tmp['first_name'][$i],
						'last_name'		=>	$tmp['last_name'][$i],
						'email'			=>	$tmp['email'][$i],
					)
				);	
			}
		}
		return $this->attendeeDetails;
	}
	
	/* **** Payment Details *** */
	
	//  Get Processor transaction ID
	public function get_processor_transaction_id() {
		return $this->row['order_processor_transaction_id'];	
	}
	
	//  Set the processor transaction ID
	public function set_processor_transaction_id($txnId) {
		$orderNum = $this->complete(); 
		$fields = array(
			'order_status'						=>	'C',
			'order_processor_transaction_id'	=>	$txnId
		);
		$result = $this->save($fields);	
		return $orderNum;
	}
	
	//  Whether payment is complete
	public function payment_already_processed() {
		$txnId = $this->get_processor_transaction_id();
		return !empty($txnId);
	}
	
	/* **** EVENT *** */
	
	//  Get the devent
	public function get_event($cache_call=true) {
		$this->_set_event($cache_call);
		return $this->event;
	}
	
	//  Get the event ID
	public function get_event_id() {
		return $this->row['order_event_id'];
	}

	public function get_track_id() {
		return $this->row['order_track_id'];
	}

	public function get_track() {
		$this->fetch_track();
		return $this->track;
	}

	//  Get the event dates
	public function get_event_dates($classroom = false) {
		$this->_set_event();
		return $this->event->get_dates($classroom);
	}
	
	//  Get the event times
	public function get_event_times() {
		$this->_set_event();
		return $this->event->get_times();
	}

	//  Get the event location
	public function get_event_location() {
		$this->_set_event();
		return $this->event->get_location();
	}
	
	//  Get the event location region
	public function get_event_region_id() {
		$this->_set_event();
		return $this->event->get_region_id();
	}
	
	//  Get the event prices
	public function get_event_prices() {
		$this->_set_event();
		return $this->event->get_prices();	
	}

	public function get_discount() {
		if(property_exists($this, 'discount')) {
			return $this->discount;
		}

		$this->discount = null;
		$discounts = Administrate::api()->raw_call('getAllDiscounts');

		foreach($discounts->AllDiscounts as $discount) {
			if($discount->Type == 'partner' && $discount->Code == $this->get_promo_code()) {
				$this->discount = $discount;
			}
		}

		return $this->discount;
	}

	// Get promo code discount rate
	public function get_discount_rate() {
		$disc = $this->get_discount();
		if($disc) {
			return $disc->Rate;
		}

		return 0;
	}

	// Get promo code discount rate
	public function get_discount_type() {
		$disc = $this->get_discount();
		if($disc) {
			return $disc->PromoType;
		}

		return 0;
	}

	//  Get the event price based on order currency
	public function get_event_price() {
		$this->_set_event();
		if (is_null($this->eventPrice)) {
			$this->eventPrice = $this->event->get_price_by_currency($this->get_currency());	
		}
		return $this->eventPrice;
	}
	
	//  Get the event price based on order currency
	public function get_event_gross_price($discounted = true) { 
		$this->_set_event();
		if (is_null($this->eventPrice)) {
			$this->get_event_price();
		}
		if(is_object($this->eventPrice)) {
			$gross = $this->eventPrice->get_gross();
			if($discounted) {
				if($this->get_discount_type() == 'fixed') {
					$net = $this->eventPrice->get_net();
					$net -= $this->get_discount_rate();
					$gross = round($net * $this->get_tax_rate(), 2);
				} else {
					$gross -= round($gross * $this->get_discount_rate(), 2);
				}
			}
			return max(0, $gross);
		}
		return null;
	}
	
	//  Get the event price based on order currency
	public function get_event_net_price($discounted = true) {
		$this->_set_event();
		if (is_null($this->eventPrice)) {
			$this->get_event_price();
		}
		if(is_object($this->eventPrice)) {
			$net = $this->eventPrice->get_net();
			if($discounted) {
				if($this->get_discount_type() == 'fixed') {
					$net -= $this->get_discount_rate();
				} else {
					$net -= round($net * $this->get_discount_rate(), 2);
				}
			}
			return max(0, $net);
		}
		return null;
	}
	
	//  Get the event default currency
	public function get_event_default_currency() {
		$this->_set_event();
		return $this->event->get_default_currency();	
	}
	
	//  Get the event object from the API
	private function _set_event($cache_call=true) {
		if(!is_null($this->get_event_id())) {
			if (!property_exists($this, 'event')) {
				$invoice_address = $this->get_invoice_address_fields();

				if ($invoice_address) {
					$this->event = Administrate::api()->get_event(
						array(
							"eventID" => $this->get_event_id(),
							"InvoiceCountry" => $invoice_address["country"]
						),
						false
					);
				} else {
					$this->event = Administrate::api()->get_event(
						$this->get_event_id(),
						$cache_call
					);
				}
			}
		} else {
			$this->event = null;
		}
	}

	public function fetch_track() {
		if(is_null($this->track)) {
			$this->track = Administrate::api()->get_learning_track(
				$this->get_track_id()
			);
		}
	}

	/* **** COURSE *** */
	
	//  Get the course code
	public function get_course_code() {
		$this->_set_course();
		return $this->course->get_code();	
	}
	
	//  Get the course title
	public function get_course_title() {
		$this->_set_course();
		return $this->course->get_title();	
	}
	
	//  Get the course summary
	public function get_course_summary() {
		$this->_set_course();
		return $this->course->get_summary();	
	}
	
	//  Get the course schedule
	public function get_course_schedule() {
		$this->_set_course();
		return $this->course->get_schedule();	
	}

	//  Get the course inclusions
	public function get_course_inclusions() {
		$this->_set_course();
		return $this->course->get_inclusions();
	}

	//  Get the course custom_text_8
	public function get_course_custom_text_8() {
		$this->_set_course();
		return $this->course->get_custom_text_8();
	}

	//  Get the course method
	public function get_course_method() {
		$this->_set_course();
		return $this->course->get_method();
	}
	
	//  Get the course prerequisites
	public function get_course_prerequisites() {
		$this->_set_course();
		return $this->course->get_prerequisites();	
	}
	
	//  Get the course topics
	public function get_course_topics() {
		$this->_set_course();
		return $this->course->get_topics();	
	}
	
	//  Get the course benefits
	public function get_course_benefits() {
		$this->_set_course();
		return $this->course->get_benefits();	
	}
	
	//  Get the course duration
	public function get_course_duration() {
		$this->_set_course();
		return $this->course->get_duration();	
	}
	
	//  Set the course
	private function _set_course() {
		$this->_set_event();
		if (!is_null($this->event)) {
			$this->course = Administrate::api()->get_course($this->event->get_course_code());
		} elseif(!property_exists($this, 'event')) {
			$this->course = null;
		}
	}
	
	/* **** API *** */
	
	//  Get the API invoice ID
	public function get_api_invoice_id() {
		return $this->row['order_api_invoice_id'];	
	}
	
	/* **** LOGS *** */
	
	//  Get the order logs
	public function get_logs() {
		require_once($this->service->get_path('/controllers/widgets/checkout/AdministrateWidgetCheckoutLog.php'));
		$where = array(
			'log_order_id'	=>	$this->get_id()
		);
		$order = array(
			'log_id'	=>	'ASC'
		);
		$dao = new AdministrateWidgetCheckoutLog($this->service);
		return $dao->get_all($where, $order);	
	}


	public function is_free() {
		if($this->get_event_id()) {
			return $this->is_event_and_mapped_items_free();
		}
		if ($this->orderIsForLearningTrackAndLearningTrackIsFree()) {
			return true;
		}
		return false;
	}

	private function are_mapped_items_free() {
		$mapped_item_price = 0;
		$mapped_items = $this->get_mapped_items_and_prices();

		foreach ($mapped_items as $item) {
			$mapped_item_price += $item['subtotal'];
		}
		return (bool) ( $mapped_item_price < 0.001 );
	}

	private function is_event_free() {
		return $this->get_event_price() ? $this->get_event_price()->is_zero() : true;
	}

	private function orderIsForLearningTrackAndLearningTrackIsFree() {
		$track_price = AdministrateLearningTrackWidget::get_price($this->get_track());
		if ($track_price) {
			return $track_price->amount == 0;
		}
		return false;
	}

	private function is_event_and_mapped_items_free() {
		if ($this->is_event_free() && $this->are_mapped_items_free()) {
			return true;
		} else {
			return false;
		}
	}

}
