<?php
//  PayPal Administrate Widget Checkout Payment Processor
class AdministrateWidgetCheckoutPaymentProcessorPayPal extends AdministrateWidgetCheckoutPaymentProcessor {

	//  Properties
	protected $key = 'paypal';
	protected $apiUrl = '';

	//  Constructor
	public function __construct(&$service, &$widget, &$order = false) {
		$this->display_name = "PayPal";

		//  Call parent class
		parent::__construct($service, $widget, $order);

		//  If the order isn't defined yet and an item number is in the request, initialize order
		if (!$this->order && isset($_REQUEST['item_number'])) {
			$this->set_order($this->get_order());
		}

		if($this->service->get_option('paypal_mode') == 'Sandbox') {
			$this->apiUrl = 'https://api-3t.sandbox.paypal.com/nvp';
		} else {
			$this->apiUrl = 'https://api-3t.paypal.com/nvp';
		}
	}

	//  Display payment form
	public function display_form() {

		// Get the config
		$config = Administrate::api()->get_payment_config( 'paypal', false);

		$https = false;
		if(isset($_SERVER['HTTPS'])) {
			if ($_SERVER['HTTPS'] == "on") {
				$https = true;
			}
		}

		$custom = array(
			'paypal_mode' => strtolower($this->service->get_option('paypal_mode')),
			'api_key' => strtolower($this->service->get_option('api_key')),
			'use_sales_opportunities' => strtolower($this->service->get_option('use_sales_opportunities'))
		);

		//  Set the params used to generate the PayPal button
		$params = array(
			"METHOD"		=>	"BMCreateButton",
			"VERSION"		=>	"65.2",
			"USER"			=>	$config->username,
			"PWD"			=>	$config->password,
			"SIGNATURE"	=>	$config->apisignature,
			"BUTTONCODE"	=>	"ENCRYPTED",
			"BUTTONTYPE"	=>	"BUYNOW",
			"BUTTONSUBTYPE"	=>	"SERVICES",
			"BUTTONCOUNTRY"	=>	"GB",
			"BUTTONIMAGE"	=>	"reg",
			"BUYNOWTEXT"	=>	"BUYNOW",
			"L_BUTTONVAR1"	=>	"item_number=" . $this->order->get_id(),
			"L_BUTTONVAR4"	=>	"quantity=" . $this->order->get_num_attendees(),
			"L_BUTTONVAR5"	=>	"currency_code=" . $this->order->get_currency(),
			"L_BUTTONVAR6"	=>	"no_shipping=1",
			"L_BUTTONVAR7"	=>	"no_note=1",
			"L_BUTTONVAR8"	=>	"notify_url=" . 'http' . ($https ? 's' : '') . '://' . $_SERVER['HTTP_HOST'] . '/common/processors/paypal/paypal_ipn_handler_weblink.php',		// IPN
			"L_BUTTONVAR9"	=>	"cancel_return=" . preg_replace('/step=[0-9]{1}/', 'step=1', $_REQUEST['cancel_base_url']),		// Cancel
			"L_BUTTONVAR10"	=>	"return=" . preg_replace('/step=[0-9]{1}/', 'step=5', $_REQUEST['cancel_base_url']),					// After completion
			"L_BUTTONVAR11"	=>	"rm=2",
			"L_BUTTONVAR12"	=>	"custom=" . urlencode(serialize($custom)),
		);

		if($this->order->get_event_id()) {
			$mapped_items = $this->order->get_mapped_items_and_prices();
			$item_amount = 0;
			foreach($mapped_items as $item) {
				$item_amount += $item['subtotal'] + $item['tax'] - $item['discount'];
			}
			$params["L_BUTTONVAR2"] = "item_name=" . $this->order->get_course_title();
			$params["L_BUTTONVAR3"] = "amount=" . (($this->order->get_event_gross_price()) + $item_amount);
		} else {
			$track = $this->order->get_track();
			$params["L_BUTTONVAR2"] = "item_name=" . $track->name;

			list(
				$tax_amount,
				$tax_name,
				$net_price,
				$discounted_net_price,
				$gross_price,
				$discounted_gross_price,
				) = AdministrateLearningTrackWidget::price_detail(
				$track,
				$this->service->get_option('currency', 'pricing'),
				$this->order->get_num_attendees(),
				$this->get_order()->get_discount_type(),
				$this->get_order()->get_discount_rate()
			);

			$params["L_BUTTONVAR3"] = "amount=" . $discounted_gross_price / $this->order->get_num_attendees();
		}


		//  Get the button variables from PayPal
		$curl = curl_init();
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($curl, CURLOPT_URL, $this->apiUrl);
		curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($params));

		$response = curl_exec($curl);
		$errors = curl_error($curl);
		curl_close($curl);

		$output = array();
		parse_str($response, $output);

		//  If the button generation was successful, display the PayPal form
		if (isset($output['ACK']) && ($output['ACK'] == 'Success')) {
			require_once($this->widget->get_path('/views/checkout_step4_payment_paypal.php'));
		} else {
			$error = ($response == false ? var_dump($errors) :  $output['L_LONGMESSAGE0']);
			$this->service->log('Unable to access PayPal button: ' . $error);
		}

	}


	//  Process the payment
	protected function _process_payment($txnId=null) {
		return parent::_process_payment($_REQUEST['txn_id']);
	}

	//  Set the order
	public function get_order() {
		if (!$this->order || !property_exists($this, 'order')) {
			$where = array(
				'order_id'			=>	$_REQUEST['item_number'],
				'order_session_id'	=>	$_REQUEST['custom']
			);
			$this->order = new AdministrateWidgetCheckoutOrder($this->service, $where);
		}
		return $this->order;
	}

	// Is order key present in the request
	public static function has_order_key() {
		return isset($_REQUEST["item_number"]);
	}
}
?>
