
// Construct monitoring report
//
var AdministrateMonitoringReport = function($) {
	var self = this;
	
	self.report          = $('#administrate_monitor_report');
	self.report_header   = $('#administrate_monitor_report_header');
	self.report_progress = $('#administrate_monitor_progress');
	self.report_generate = $('#generate_report');
	self.report_send     = $('#send_report');
	self.user_email      = $('#user_email');
	
	self.url = $('#administrate_monitor_url').val();
	
	self.report_generate.click(function(e) {
		e.preventDefault();
		
		$(this).attr("disabled", "disabled");
		$(this).attr("value", "Please Wait");
		
		self.progress_bar($);
		self.get_json($);
	});
	
	self.report_send.click(function(e) {
		if( !self.validate_email( self.user_email.val() ) ) {
			
			self.user_email.addClass("invalid_email");
			setTimeout(function() { self.user_email.removeClass("invalid_email"); }, 500);
			e.preventDefault();
		}
	});
};

// Progress bar
//
AdministrateMonitoringReport.prototype.progress_bar = function($) {
	var self = this;
	var cnt = 0;
	
	self.report_progress.html('');
	
	self.spinner_id = setInterval(
		function() {
			if(cnt++ < 50 ) {
				self.report_progress.append(' .');
			}
			else {
				self.report_progress.html('');
				cnt = 0;
			}
		},
		400 // .4 second
	);
};

// Get report summary JSON
//
AdministrateMonitoringReport.prototype.get_json = function($) {
	var self = this;
	
	$.getJSON(
		self.url, 
		function(report_summary) {
			clearInterval( self.spinner_id );
			
			var r_json = $.parseJSON(report_summary)
			
			$("#administrate_monitor_submit").delay(300).fadeOut(500, // .5 second
				function() {
					$("#administrate_monitor_send").fadeIn(400);
					self.report.animate( {height: 1400}, 800 );
					
					$.each( r_json.sections, function( section_name, section ) {
						self.section($, section_name, section);
					});
					
					$(".administrate_report_rerun").on("click", function(e) {
						e.preventDefault();
						self.progress_bar($);
						self.get_json($);
						
						$(".administrate_r_summary_section").hide();
						$("#administrate_monitor_send").hide();
						self.report.css("height", "150px");
						$("#administrate_monitor_submit p").html("Generating Report");
						$("#administrate_monitor_submit").show();
					});
			});
		}
	).fail( function() {
		self.report.html('<p>A problem has occured, please <a href="?page=administrate_monitoring">try again.</a></p>');
	});
};

// Add report section
//
AdministrateMonitoringReport.prototype.section = function($, section_name, section) {
	var self = this;
	
	var color_code = self.get_color_code(section.grade);
	var item  = $("<div></div>").addClass("administrate_r_summary_section");
	var grade = $("<div></div>").addClass("administrate_r_summary_grade").html( "<div>"+color_code[0]+"</div>" );
	var text  = $("<div>"+section_name+"</div>").addClass("administrate_r_summary_text");
	
	$.each( section.notes, function( index, note ) {
		var note_p  = $('<p>'+note+'</p>');
		text.append(note_p);
	});
	
	grade.find("div").css("background-color", color_code[1] );
	item.append(grade);
	item.append(text);
	item.appendTo(self.report);
}

// RGB color for benchmark gade
//
AdministrateMonitoringReport.prototype.get_color_code = function(grade) {
	var green  = "rgb(50, 170, 50)";
	var yellow = "rgb(230, 230, 0)";	
	var red    = "rgb(170, 50, 50)";
	
	if( grade >= 70  ) {
		return ["&#xf147;", green];
	}
	else if ( grade >= 50 ) {
		return ["&#xf460;", yellow];
	}
	
	return ["&#xf335;", red];
}

// Validate user e-mail
//
AdministrateMonitoringReport.prototype.validate_email = function(email) { 
	return /^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/.test(email);
} 
