<?php
//  SagePay Administrate Widget Checkout Payment Processor
class AdministrateWidgetCheckoutPaymentProcessorSagePay extends AdministrateWidgetCheckoutPaymentProcessor {
	
	protected $key = 'sagepay';
	protected $apiUrl = '';
	protected $decryptedParams = array();
	
	//  Constructor
	public function __construct(&$plugin, &$widget, &$order = false) {
		$this->display_name = "SagePay";
		
		//  Call parent class
		parent::__construct($plugin, $widget, $order);
		
		//  Parse the params
		if (isset($_GET['crypt'])) {
			parse_str($this->_decrypt_str($_GET['crypt']), $this->decryptedParams);
		}
		
		//  If the order isn't defined yet and an item number is in the request, initialize order
		if (!$this->order && isset($this->decryptedParams['VendorTxCode'])) {
			$this->set_order($this->get_order());
		}
		
		if($this->plugin->get_option('sagepay_mode', 'checkout') == 'Test') {
			$this->apiUrl = 'https://test.sagepay.com/gateway/service/vspserver-register.vsp';
		} else {
			$this->apiUrl = 'https://live.sagepay.com/gateway/service/vspserver-register.vsp';
		}
		
		$this->protocolVer = "3.00";
		
		if($this->service->get_option('sagepay_mode', 'checkout') == 'Simulator') {
			// Testing simulator is only available with v2.23
			$this->protocolVer = "2.23";
			$this->apiUrl = 'https://test.sagepay.com/Simulator/VSPFormGateway.asp';
		}
	}
	
	//  Display payment form
	public function display_form() {
		
		//  Get the required options
		$vendor = $this->plugin->get_option('sagepay_vendor_username', 'checkout');
		$emailMessage = $this->plugin->get_option('sagepay_email_message', 'checkout');
		$notifyUser = $this->plugin->to_boolean($this->plugin->get_option('cc_send_email', 'checkout'));
		
		// Use the session ID for the transaction Code. Append a timestamp
		// so that TX code is changed if we cancel then re-submit the form.
		$txCode= $this->plugin->get_namespace() . '-wp-' . $this->order->get_id() . '-' . time();
		
		//  Figure out what emails to send
		if (!$notifyUser) {
			$sendEmail = 0;	
		} else {
			$sendEmail = 1;
		}
		
		//  Generate the basket string
		$track_total = 0;
		$track = null;
		if($this->order->get_event_id()) {
			$basketParts = array(
				1,                                                                                        // always 1 line of detail
				str_replace(":", " ", str_replace("&", " ", $this->order->get_course_title())),           // item name
				$this->order->get_num_attendees(),                                                        // quantity
				$this->order->get_event_net_price(),                                                      // price excluding tax
				$this->order->get_event_gross_price() - $this->order->get_event_net_price(),              // tax
				$this->order->get_event_gross_price(),                                                    // price including tax
				$this->order->get_event_gross_price() * $this->order->get_num_attendees()                  // total
			);
		} else {
			$track = $this->order->get_track();

			list(
				$tax_amount,
				$tax_name,
				$net_price,
				$discounted_net_price,
				$gross_price,
				$discounted_gross_price,
				) = AdministrateLearningTrackWidget::price_detail(
				$track,
				$this->service->get_option('currency', 'pricing'),
				$this->order->get_num_attendees(),
				$this->get_order()->get_discount_type(),
				$this->get_order()->get_discount_rate()
			);

			$basketParts = array(
				1,
				str_replace(":", " ", str_replace("&", " ", $track->name)),
				$this->order->get_num_attendees(),
				$discounted_net_price / $this->order->get_num_attendees(),
				$tax_amount / $this->order->get_num_attendees(),
				$discounted_gross_price / $this->order->get_num_attendees(),
				$discounted_gross_price
			);

			$track_total = $discounted_gross_price;
		}
		$basket = implode(':', $basketParts);

		//  If the billing country is UK, change to GB
		if ($this->order->get_invoice_country() == 'UK') {
			$country = 'GB';
		} else {
			$country = $this->order->get_invoice_country();
		}

		$territory = '';
		//  If the billing country is US, SagePay requires the state abbreviation to be 2 characters (sigh)
		if ($this->order->get_invoice_country() == 'US') {
			
			//  Strip everything but characters
			$territory = preg_replace("/[^A-Za-z ]/", '', $this->order->get_invoice_territory());
			
			//  If we don't have a match already, search for the full state name
			if (!isset($this->usStates[strtoupper($territory)])) {
				if ($tmpKey = array_search(strtolower($territory), $this->usStates)) {
					$territory = $tmpKey;
				}

			//  Otherwise just make sure the state is upper case
			} else {
				$territory = strtoupper($territory);
			}
		}

		global $_ADMINISTRATE_CONFIG;
		$notify_url = $_ADMINISTRATE_CONFIG['plugin']['url'] . 'common/processors/sagepay/sagepay_ipn_handler.php';

		//  Set the parameters to pass
		$params = array(
			'VPSProtocol'   => $this->protocolVer,
			'Vendor'        => $vendor,
			'VendorTxCode'  => $txCode,
			'TxType'        => 'PAYMENT',

			//  Transaction fields
			'RedirectionURL'	=>	$this->widget->get_step_url(5, true),
			'FailureURL'		=>	$this->widget->get_step_url(4, true),
			'NotificationURL'	=>	$notify_url,
			
			//  Order info
			'Currency'			=>	$this->order->get_currency(),
			'CustomerName'		=>	$this->order->get_buyer_first_name() . ' ' . $this->order->get_buyer_last_name(),
			'Basket'			=>	$basket,
			
			//  Email options
			'SendEMail'			=>	$sendEmail,
			'CustomerEMail'		=>	$this->order->get_buyer_email(),
			//'VendorEMail'		=>	$adminEmail,
			'eMailMessage'		=>	$emailMessage,
			
			//  Billing fields
			'BillingFirstnames'	=>	substr($this->order->get_buyer_first_name(), 0, 20),
			'BillingSurname'	=>	substr($this->order->get_buyer_last_name(), 0, 20),
			'BillingAddress1'	=>	$this->order->get_invoice_street(),
			//'BillingAddress2'	=>	'',
			'BillingCity'		=>	$this->order->get_invoice_city(),
			'BillingPostCode'	=>	$this->order->get_invoice_postal_code(),
			'BillingCountry'	=>	$country,
			'BillingState'		=>	$territory,
			'BillingPhone'		=>	$this->order->get_buyer_phone(),
			
			//  Shipping fields
			'DeliveryFirstnames'=>	substr($this->order->get_buyer_first_name(), 0, 20),
			'DeliverySurname'	=>	substr($this->order->get_buyer_last_name(), 0, 20),
			'DeliveryAddress1'	=>	$this->order->get_invoice_street(),
			//'DeliveryAddress2'	=>	'',
			'DeliveryCity'		=>	$this->order->get_invoice_city(),
			'DeliveryPostCode'	=>	$this->order->get_invoice_postal_code(),
			'DeliveryCountry'	=>	$country,
			'DeliveryState'		=>	$territory,
			'DeliveryPhone'		=>	$this->order->get_buyer_phone(),
			
			//  Payment options
			'AllowGiftAid'		=>	0,
			'ApplyAVSCV2'		=>	1,
			'Apply3DSecure'		=>	1
		
		);

		if($this->order->get_event_id()) {
			$mapped_items = $this->order->get_mapped_items_and_prices();
			$item_amount = 0;
			foreach($mapped_items as $item) {
				$item_amount += $item['subtotal'] + $item['tax'] - $item['discount'];
			}
			$params['Amount'] = floatval(($this->order->get_event_gross_price() * $this->order->get_num_attendees()) + $item_amount);
			$params['Description'] = substr($this->order->get_course_title() . ' ' . __('Course Registration', 'administrate'), 0, 99);
		} else {
			$params['Amount'] = floatval($track_total);
			$params['Description'] = substr($track->description, 0, 99);
		}

		$curlSession = curl_init();

		curl_setopt($curlSession, CURLOPT_URL, $this->apiUrl);
		curl_setopt($curlSession, CURLOPT_HEADER, 0);
		curl_setopt($curlSession, CURLOPT_POST, 1);
		curl_setopt($curlSession, CURLOPT_POSTFIELDS, $this->_arrayToQueryString($params));
		curl_setopt($curlSession, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curlSession, CURLOPT_TIMEOUT, 30);
		curl_setopt($curlSession, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($curlSession, CURLOPT_SSL_VERIFYPEER, false);

		$raw_response = curl_exec($curlSession);

		$curl_info = curl_getinfo($curlSession);
		curl_close($curlSession);

		$response = $this->_queryStringToArray($raw_response, "\r\n");

		if($curl_info['http_code'] != 200 || $response['Status'] == 'INVALID') {
			require_once($this->widget->get_path('/views/checkout_step4_unable_to_contact_payment_processor.php'));
		} else {
			$this->order->set_transaction_id($response['VPSTxId']);

			//  Include the form
			require_once($this->widget->get_path('/views/checkout_step4_payment_sagepay.php'));
		}
	}

	//  Get the encryption password
	protected function _get_password() {
		if (!property_exists($this, 'password')) {
			$this->password = $this->plugin->get_option('sagepay_encryption_password', 'checkout');
		}
		return $this->password;
	}
		
	//  Validate payment
	protected function _payment_has_errors() {
		
		//  Check for errors with the payment
		$errors = false;
		
		//  If the return status is not 'OK', there was a problem
		if ($this->decryptedParams['Status'] != 'OK') {
			$errors = array();
			if ($this->decryptedParams['Status'] == 'NOTAUTHED') {
				array_push($errors, __('It appears that the payment was rejected by your bank.', 'administrate'));
			} else if ($this->decryptedParams['Status'] == 'ABORT') {
				array_push($errors, __('Payment was canceled.', 'administrate'));
			} else if ($this->decryptedParams['Status'] == 'REJECTED') {
				array_push($errors, __('Sorry, your payment was rejected.', 'administrate'));
			} else if ($this->decryptedParams['Status'] == 'ERROR') {
				array_push($errors, __('Sorry, and error occurred. Please try again later or contact us.', 'administrate'));
			}
		}
		
		//  Return the errors
		return $errors;	
		
	}
	
	//  Process the payment
	protected function _process_payment($txnId=false) {
		
		//  Save the transaction ID
		parent::_process_payment($this->decryptedParams['VPSTxId']);
		
		//  Redirect to step 5 (completion)
		wp_redirect($this->widget->get_step_url(5, true));
		exit;
		
	}
	
	//  Set the order
	public function get_order() {
		if (!$this->order || !property_exists($this, 'order')) {
			$txParts = explode('-', $this->decryptedParams['VendorTxCode']);
			$where = array(
				'order_id'	=>	$txParts[count($txParts)-1]
			);
			$this->order = new AdministrateWidgetCheckoutOrder($this->plugin, $where);	
		}
		return $this->order;
	}
	
	//  Get API URL
	private function _get_api_url() {
		return $this->apiUrl;	
	}
	
	// Is order key present in the request
	public static function has_order_key() {
		return isset($_GET['crypt']);
	}
	
	private function _get_protocol_ver() {
		return $this->protocolVer;
	}

	/**
	 * Convert a data array to a query string ready to post.
	 *
	 * @param  array   $data        The data array.
	 * @param  string  $delimeter   Delimiter used in query string
	 * @param  boolean $urlencoded  If true encode the final query string
	 *
	 * @return string The array as a string.
	 */
	private function _arrayToQueryString(array $data, $delimiter = '&', $urlencoded = false)
	{
		$queryString = '';
		$delimiterLength = strlen($delimiter);

		// Parse each value pairs and concate to query string
		foreach ($data as $name => $value)
		{
			// Apply urlencode if it is required
			if ($urlencoded)
			{
				$value = urlencode($value);
			}
			$queryString .= $name . '=' . $value . $delimiter;
		}

		// remove the last delimiter
		return substr($queryString, 0, -1 * $delimiterLength);
	}

	/**
	 * Convert string to data array.
	 *
	 * @param string  $data       Query string
	 * @param string  $delimeter  Delimiter used in query string
	 *
	 * @return array
	 */
	private function _queryStringToArray($data, $delimeter = "&")
	{
		// Explode query by delimiter
		$pairs = explode($delimeter, $data);
		$queryArray = array();

		// Explode pairs by "="
		foreach ($pairs as $pair)
		{
			$keyValue = explode('=', $pair);

			// Use first value as key
			$key = array_shift($keyValue);

			// Implode others as value for $key
			$queryArray[$key] = implode('=', $keyValue);
		}
		return $queryArray;
	}

}
