<?php
//  Administrate course table widget
class AdministrateWidgetCourseList extends WPPluginPatternWidget {

	//  Configuration
	protected $key = 'course_list';
	protected $category = false;
	protected $subcategory = false;
	
	//  Run the checkout
	public function run($params = array()) {
		
		//  Extract parameters
		extract(shortcode_atts(array(
			'category'		=>	0,
			'subcategory'	=>	0,
			'show_codes'	=>	true,
			'show_current_online' => $this->plugin->get_option('show_current_online', 'event'),
			'course_code'   => false
		), $params));

		//  Set param types
		$show_codes = (($show_codes === true) || ($show_codes === 'true'));

		$category_ids = $subcategory_ids = array();

		if($category) {
			$category_ids = array_map('intval', explode(',', $category));
		}

		if($subcategory) {
			$subcategory_ids = array_map('intval', explode(',', $subcategory));
		}

		// If no course parameter was passed, see if we can get a request parameter
		if ((empty($category_ids)) && (empty($subcategory_ids))) {

			if (isset($_REQUEST[$this->plugin->add_namespace(array('subcategory', 'id'))])) {
				$subcategory_ids = array_map('intval', explode(',', $_REQUEST[$this->plugin->add_namespace(array('subcategory', 'id'))]));
			}

			else if (isset($_REQUEST[$this->plugin->add_namespace(array('category', 'id'))])) {
				$category_ids = array_map('intval', explode(',', $_REQUEST[$this->plugin->add_namespace(array('category', 'id'))]));
			}
		}

		//  Save course page option
		$coursePage = $this->plugin->get_option('course_page', 'course');
		
		//  Initialize SOAP params
		$params = array(
			'IncludeCurrentOnline' => $show_current_online
		);

		if($course_code) {
			$params['CourseCode'] = array_map('trim', explode(',', $course_code));
		}

		if(!empty($subcategory_ids)) {
			$this->_set_courses($params, true, $subcategory_ids);
		} 
		else if(!empty($category_ids)) {
			$this->_set_courses($params, false, $category_ids);
		}
		else {
			$this->_set_courses($params);
		}
		
		//  Display the course list
		ob_start();
		include($this->get_path('/views/course_list.php'));	
		return ob_get_clean();
		
	}

	//  Set courses
	private function _set_courses($params, $is_subcategory=false, $id=null) {
		$show_empty = $this->plugin->to_boolean($this->plugin->get_option('show_empty_courses', 'course'));

		if ($is_subcategory) {
			// We make different API call when the show_empty flag is set,
			// it expect a different parameter name.
			$id_param = 'CourseSubCategoryID';
		}
		else {
			$id_param = 'CourseCategoryID';
		}

		// Add ID parameter to API call
		if ( !is_null($id) ) {
			$params[$id_param] = $id;
		}

		if ($show_empty) {
			$this->courses = Administrate::api()->get_course_list($params);
		} else {
			$this->courses = array();

			if(array_key_exists('CourseCode', $params)) {
				$params['CourseCode'] = array_map('trim', explode(',', $params['CourseCode']));
			}

			$events = Administrate::api()->get_event_list($params);
			foreach ($events as $event) {
				$key = $event->get_course_code();
				if (!isset($this->courses[$key])) {
					$course = Administrate::api()->get_course( $event->get_course_code());
					if ($course->exists()) {
						$this->courses[$key] = $course;
					}
				}
			}
			ksort($this->courses);
		}
	}

	//  Get courses
	public function get_courses() {
		return $this->courses;
	}

}
